/*	$Kolabore: xsicompat.h,v 1.2 2007/02/09 20:58:49 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __XSICOMPAT_H__
#define __XSICOMPAT_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef __cplusplus
#  include "typesetup.h"
#endif /* __cplusplus */

/* Get size_t and friends: */
#include <sys/types.h>

#ifdef HAVE_GOOD_STDINT_H
#  include <stdint.h>
#endif /* HAVE_GOOD_STDINT_H */
#ifdef HAVE_INTTYPES_H
#  include <inttypes.h>
#endif /* HAVE_INTTYPES_H */

/* Inttype.h macros that we use */
#ifndef PRIi64
#  define PRIi64 "lld"
#endif /* PRIi64 */

#ifndef PRIu64
#  define PRIu64 "llu"
#endif /* PRIu64 */

#ifndef PRIi32
#  define PRIi32 "d"
#endif /* PRIi64 */

#ifndef PRIu64
#  define PRIu32 "u"
#endif /* PRIu64 */

/* Try to get timerclear() */
#include <sys/time.h>

#ifndef timerclear
#  define timerclear(tvp)	(tvp)->tv_sec = (tvp)->tv_usec = 0
#endif /* timerclear */

/* Try to get __dead from <sys/cdefs.h> in case it isn't there already */
#ifdef HAVE_SYS_CDEFS_H
#  include <sys/cdefs.h>
#endif

/* Code related to __dead adapted from OpenBSD <sys/cdefs.h> */
#if !defined(__dead)

/*
 * Macro to test if we're using a specific version of gcc or later.
 */
#if defined(__GNUC__) && defined(__GNUC_MINOR__)
#  undef __GNUC_PREREQ__
#  define __GNUC_PREREQ__(ma, mi) \
	((__GNUC__ > (ma)) || (__GNUC__ == (ma) && __GNUC_MINOR__ >= (mi)))
#else
#  define __GNUC_PREREQ__(ma, mi) 0
#endif

/*
 * For some reason, this doesn't really work with Sun's cc/CC compiler,
 * hence the __SUNPRO_C[C] defines above
 */
#if !__GNUC_PREREQ__(2, 5)
#  define __attribute__(x)	/* delete __attribute__ if non-gcc or gcc1 */
#  if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#    define __dead	__volatile
#  elif defined(lint)
#    define __dead	/* NORETURN */
#  else
#    define __dead
#  endif
#elif !defined(__STRICT_ANSI__)
#  define __dead	__attribute__((__noreturn__))
#else
#  define __dead
#endif

#endif /* !__dead */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef HAVE_STRLCAT
#  define strlcat xsi_strlcat
#endif /* HAVE_STRLCAT */
size_t		xsi_strlcat(char *, const char *, size_t);

#ifndef HAVE_STRLCPY
#  define strlcpy xsi_strlcpy
#endif /* HAVE_STRLCPY */
size_t		xsi_strlcpy(char *, const char *, size_t);

#ifndef HAVE_STRTONUM
#  define strtonum xsi_strtonum
#endif /* HAVE_STRTONUM */
long long	xsi_strtonum(const char *, long long, long long, const char **);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XSICOMPAT_H__ */
