/*	$Kolabore: common.cpp,v 1.20 2007/02/09 13:06:06 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "log.h"

extern "C" {
#include <sys/utsname.h>
}

#include <iostream>

#include <cerrno>

#include "common.h"

using namespace std;

XSICommon::XSICommon(void)
{
	packetFilters.resize(XSI_PFILTER_MAX);
}

/* virtual dtor */
XSICommon::~XSICommon(void)
{
	// empty
}

/*
 * Devices to be opened before dropping privileges.
 * Silently returns 0 on success. Warns and returns >0 on error.
 *
 * NOTE: Valid XML must not rely on information gathered through privileged
 *       devices! Only optional, user-requested features may depend on
 *       things that may need super-user privileges.
 */
/* virtual */
int
XSICommon::tryOpenPrivilegedDevices(void)
{
	if (configuration->showRaid()) {
		cerr << "Sorry, RAID monitoring support is not (yet?) "
			"available on your OS." << endl;
		return (1);
	}

	return (0);
}

/* virtual */
int
XSICommon::initializePacketfilters(void)
{
	if (configuration->showPacketFilter()) {
		int	ret = 0;

		ret += pf.init();
		ret += ipfilter.init();
		ret += netfilter.init();

		if (ret == 0) {
			cerr << "No supported packet filter found or usable."
				<< endl;
			return (1);
		}
	}

	return (0);
}

/* virtual */
void
XSICommon::updatePacketfilterStats(void)
{
	pf.updateStats(packetFilters[XSI_PFILTER_PF]);
	ipfilter.updateStats(packetFilters[XSI_PFILTER_IPFILTER]);
	netfilter.updateStats(packetFilters[XSI_PFILTER_NETFILTER]);
}

/* virtual */
void
XSICommon::collectCommonInfo(void)
{
	struct utsname	name;

	if (uname(&name) == -1)
		XSI_Log::log()->err(__func__, errno, "uname");

	system.setArch(name.machine);
	system.setOSType(name.sysname);
	system.setOSRelease(name.release);
}

/* virtual */
void
XSICommon::updateCommonStats(void)
{
	// empty
}

/* virtual */
void
XSICommon::collectSysInfo(void)
{
}

/* virtual */
void
XSICommon::updateStats(void)
{
}
