/*	$Kolabore: linux.h,v 1.46 2007/02/09 15:50:43 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __LINUX_H__
#define __LINUX_H__

#include "common.h"

enum linux_sensor_type {
	UNKNOWN = 0,
	VOLT,
	TEMP,
	FAN,
	CPUVOLT,
	AMP,
	OTHER
};

struct linux_sensor_private {
	string		driver;
	string		name;
	enum linux_sensor_type type;
	int64_t 	value;
	int		id;
	int		alarm;
	bool		has_fault;
};

using namespace std;

struct linux_private;

class XSI : public XSICommon {
public:
	XSI(void);
	~XSI(void);

private:
	void		collectSysInfo(void);
	void		updateStats(void);

	void		linuxInit(void);

	void		readSystemNumCPUs(void);
	void		readSystemOSInfo(void);

	void		updateCPUStats(void);
	void		updateSystemCPUFreq(void);
	void		updateSystemSysInfo(void);
	void		updateLoadAvg(void);
	void		updateProcInfo(void);
	void		updateInterrupts(void);
	void		updateSensors(void);
	void		updateFSStats(void);
	void		updateDiskStats(void);
	void		updateNetStats(void);
	void		updateDefaultRoutes(void);

	void		getNetIfInfo(XSI_NetIfStat &, struct ifaddrs *);
	void		getNetIfAddrs(XSI_NetIfStat &, struct ifaddrs *, string);
	void		getNetIfCounters(XSI_NetIfStat &, vector<string>, string);
	void		getNetIfStatus(XSI_NetIfStat &, string);

	uint8_t *	getNetlinkMessage(int, size_t &);
	int		parseRoutes(uint8_t *, size_t, struct in_addr *);
	vector<struct linux_sensor_private> getSensors(vector<string>);
	enum linux_sensor_type getSensorType(string);
	void		getSensorID(string, int &);
	string		getSysfsDeviceName(const char *);
	string		getDirName(const char *);
	string		getProcnameFromProcStat(string &);
	uint64_t	getLinuxProcCPTime(uint32_t);

	struct linux_private *priv;
	int		num_cpu;
	int64_t       **last_cp_time, **cur_cp_time;
	int		rtnl_fd;
};

#endif /* __LINUX_H__ */
