/*	$Kolabore: main.cpp,v 1.102 2007/02/09 14:32:35 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "log.h"

extern "C" {
#include <sys/param.h>
#ifdef HAVE_LINUX_PARAM_H
#  include <linux/param.h>
#endif
#include <sys/time.h>
#ifdef HAVE_SYS_RESOURCE_H
#  include <sys/resource.h>
#endif
#include <sys/socket.h>
#include <sys/stat.h>

#include <arpa/inet.h>
#include <netinet/in.h>

#include <fcntl.h>
#include <grp.h>
#include <netdb.h>
#ifdef HAVE_PATHS_H
#  include <paths.h>
#endif
#include <poll.h>
#include <pwd.h>
#include <signal.h>	// Use C header, <csignal> is broken with Solaris' CC.
#include <unistd.h>

#ifdef HAVE_ZLIB_H
#  include <zlib.h>
#endif

#include <libxml/tree.h>
}

#include <iostream>
#include <string>
#include <vector>

#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "cfg.h"
#include "interface.h"
#include "server.h"

/* For Solaris, possibly others (usually defined in <paths.h>.) */
#ifndef _PATH_DEVNULL
#  define _PATH_DEVNULL "/dev/null"
#endif /* _PATH_DEVNULL */

#define _XSI_MAGIC_BEGIN	"XSICMPR("
#define _XSI_MAGIC_END		")---"
#define _XSI_MAGIC_LENGTH	((strlen(_XSI_MAGIC_BEGIN) + sizeof(uint32_t) \
					+ strlen(_XSI_MAGIC_END)))

#define SIGS_INSTALL(a, b)	do {					\
	unsigned int i; 						\
	for (i = 0; i < sizeof(a) / sizeof(int); i++) { 		\
		if (sigaction((a)[i], (b), NULL) == -1) {		\
			cerr << "Error: sigaction: " << strerror(errno) \
				<< endl;				\
			return (1);					\
		}							\
	}								\
} while (0);
#define SIGS_SUSPEND(a, b)	do {					\
	unsigned int	i;						\
	sigset_t	sigset; 					\
	sigemptyset(&sigset);						\
	for (i = 0; i < sizeof(a) / sizeof(int); i++) { 		\
		if (sigaddset(&sigset, (a)[i]) == -1)			\
			XSI_Log::log()->err(__func__, errno, "sigaddset"); \
	}                                                       	\
	if (sigprocmask(SIG_BLOCK, &sigset, (b)) == -1)			\
		XSI_Log::log()->err(__func__, errno, "sigprocmask");	\
} while (0);
#define SIGS_RESTORE(a) 	do {					\
	if (sigprocmask(SIG_SETMASK, (a), NULL) == -1)			\
		XSI_Log::log()->err(__func__, errno, "sigprocmask");	\
} while (0);

extern int	optind;
extern int	errno;

#ifndef HAVE___PROGNAME
char *__progname;
#endif /* !HAVE___PROGNAME */

volatile sig_atomic_t	 update = 0;
volatile sig_atomic_t	   quit = 0;
volatile sig_atomic_t	 reread = 0;
volatile sig_atomic_t	 pstats = 0;
volatile sig_atomic_t	s_error = 0;
const int		xsi_signals[] = { SIGTERM, SIGINT, SIGALRM, SIGHUP,
					  SIGUSR1 };
const int		xsi_crash_sigs[] = { SIGILL, SIGFPE, SIGBUS, SIGSEGV,
					     SIGSYS };

extern "C" {
	static void	sighdlr(int);
	static void	crashhdlr(int);
}
static unsigned char *	zCompress(xmlChar *, size_t, unsigned long *);
static void		zFree(unsigned char *);
static char *		xsi_strdup(const char *);
static char *		xsi_getProgname(char *);
static void		xsi_daemonize(void);

using namespace std;

static void
sighdlr(int sig)
{
	switch (sig) {
	case SIGTERM:
	case SIGINT:
		quit = 1;
		break;
	case SIGALRM:
		update = 1;
		break;
	case SIGHUP:
		reread = 1;
		break;
	case SIGUSR1:
		pstats = 1;
		break;
	default:
		s_error = sig;
		break;
	}
}

static void
crashhdlr(int sig)
{
	sigset_t	sigset;

	/*
	 * No more signals from hereon (SIGKILL and SIGSTOP excluded), this is
	 * the end. Bye-bye!
	 */
	sigfillset(&sigset);
	sigprocmask(SIG_BLOCK, &sigset, NULL);

	XSI_Log::log()->crash(sig);
}

/*
 * Wrapper for zlib's compress() function
 */
static unsigned char *
zCompress(xmlChar *src, size_t sLen, unsigned long *zLen)
{
	unsigned char  *dest = NULL;

#ifdef HAVE_WORKING_ZLIB
	unsigned long	dLen = compressBound(sLen);
	int		ret;
	unsigned char  *dp;
	size_t		bufsiz;

	if (sLen > UINT32_MAX) {
		XSI_Log::log()->warn(__func__, "Source buffer too large");
		return (NULL);
	}

	bufsiz = (size_t)dLen + _XSI_MAGIC_LENGTH;
	if ((dest = new unsigned char[bufsiz]) == NULL)
		XSI_Log::log()->err(__func__, "Out of memory");

	dp = dest;

	memcpy(dp, _XSI_MAGIC_BEGIN, strlen(_XSI_MAGIC_BEGIN));
	dp += strlen(_XSI_MAGIC_BEGIN);

	*((uint32_t *)dp) = (uint32_t)htonl((uint32_t)sLen);
	dp += sizeof(uint32_t);

	memcpy(dp, _XSI_MAGIC_END, strlen(_XSI_MAGIC_END));
	dp += strlen(_XSI_MAGIC_END);

	if ((ret = compress((Bytef *)dp, (uLongf *)&dLen, (Bytef *)src, sLen)) != Z_OK) {
		switch (ret) {
		case Z_MEM_ERROR:
			XSI_Log::log()->warn(__func__, "compress: Out of memory");
			delete[] dest;
			return (NULL);
		case Z_BUF_ERROR:
			XSI_Log::log()->warn(__func__, "compress: Destination buffer too small");
			delete[] dest;
			return (NULL);
		default:
			XSI_Log::log()->warn(__func__, "compress: Unknown error");
			delete[] dest;
			return (NULL);
		}
	}

	*zLen = dLen + _XSI_MAGIC_LENGTH;
#endif /* HAVE_WORKING_ZLIB */

	return (dest);
}

static void
zFree(unsigned char *buf)
{
	if (buf != NULL) {
		delete[] buf;
		buf = NULL;
	}
}

static char *
xsi_strdup(const char *s)
{
	char   *c;
	size_t	siz = strlen(s) + 1;

	if ((c = new char[siz]) == NULL) {
		cerr << "Error: Out of memory." << endl;
		exit(1);
	}
	strlcpy(c, s, siz);

	return(c);
}

/* Borrowed from OpenNTPd-portable's compat-openbsd/bsd-misc.c */
static char *
xsi_getProgname(char *argv0)
{
#ifdef HAVE___PROGNAME
	extern char *__progname;

	return (xsi_strdup(__progname));
#else
	char *p;

	if (argv0 == NULL)
		return (const_cast<char *>("xsi"));
	p = strrchr(argv0, '/');
	if (p == NULL)
		p = argv0;
	else
		p++;

	return (xsi_strdup(p));
#endif /* HAVE___PROGNAME */
}

/*
 * Modelled after 4.4BSD's daemon(3) (OpenBSD's daemon.c, to be exact,)
 * equivalent to daemon(1, 0);
 */
static void
xsi_daemonize(void)
{
	int	fd;

	switch (fork()) {
	case -1:
		cerr << "Error: fork: " << strerror(errno) << endl;
		exit(1);
	case 0:
		break;
	default:
		_exit(0);
	}

	if (setsid() == -1)
		XSI_Log::log()->err(__func__, errno, "setsid");

	if ((fd = open(_PATH_DEVNULL, O_RDWR, 0)) != -1) {
		dup2(fd, STDIN_FILENO);
		dup2(fd, STDOUT_FILENO);
		dup2(fd, STDERR_FILENO);
		if (fd > 2)
			close(fd);
	}
}

int
main(int argc, char **argv)
{
	Config		       *cfg = NULL;
	Server		       *server = NULL;
	XSI		       *sysinfo = NULL;
	struct sigaction	act;
	xmlDocPtr		xmlDocument = NULL;
	xmlChar 	       *buffer = NULL;
	unsigned char	       *zBuffer = NULL;
	vector<struct client>	clients;
	struct itimerval	updateTimer;
	struct timeval		xsi_starttime;
	struct passwd	       *passwd;
	struct group	       *group;
	int			error = 0;
	uint64_t		request_counter = 0;
#if defined(INSANE_OS)
	const bool		insane = true;
#else
	const bool		insane = false;
#endif /* INSANE_OS */
	extern char	       *__progname;

	__progname = xsi_getProgname(argv[0]);

	/*
	 * Initialization and setup:
	 */
	cfg = new Config(argc, argv);
	sysinfo = new XSI;
	sysinfo->XSIBase_init(cfg);
	LIBXML_TEST_VERSION;
#ifdef HAVE_WORKING_ZLIB
	if (strncmp(zlibVersion(), ZLIB_VERSION, 1) != 0) {
		cerr << "Zlib version mismatch, please recompile this software for your system."
			<< endl;
		error = 1;
		goto shutdown_label;
	}
#else
	if (cfg->compressXml()) {
		cerr << "Sorry, your system's libz is too old or compression support was explicitly\n"
			<< "disabled. Install a recent libz that offers compressBound() and recompile\n"
			<< "this program." << endl;
		error = 1;
		goto shutdown_label;
	}
#endif /* HAVE_WORKING_ZLIB */
	if (sysinfo->tryOpenPrivilegedDevices() != 0 ||
	    sysinfo->initializePacketfilters() != 0) {
		cerr << "Aborted -- revise command line parameters." << endl;
		error = 2;
		goto shutdown_label;
	}

	if (gettimeofday(&xsi_starttime, NULL) == -1) {
		cerr << "Error: gettimeofday: " << strerror(errno) << endl;
		error = 1;
		goto shutdown_label;
	}

	/*
	 * In One-Shot Mode, we're done quickly:
	 */
	if (cfg->oneShot()) {
		sysinfo->readSysInfo();
		sysinfo->buildSysInfoDoc();
		xmlDocument = sysinfo->getSysInfoDocPtr();
		xmlSaveFormatFileEnc("-", xmlDocument, "UTF-8", 1);
		error = 0;
		goto shutdown_label;
	}

	/*
	 * Set up the server:
	 */
	server = new Server(cfg);
	sysinfo->setListenAddrs(server->getListenAddrStrings());
	sysinfo->setListenService(server->getListenServiceString());

	/*
	 * Chroot, if possible, and drop privileges:
	 */
	if (geteuid() == 0) {
		const char *gp = cfg->getPrivdropGroup();
		gid_t gid;

		if ((passwd = getpwnam(cfg->getPrivdropUser())) == NULL) {
			cerr << "Error: getpwnam: unknown user `" <<
				cfg->getPrivdropUser() << "'" << endl;
			error = 1;
			goto shutdown_label;
		}

		if (passwd->pw_uid == 0) {
			cerr << "Error: privdrop user has superuser privileges."
				<< endl;
			error = 1;
			goto shutdown_label;
		}

		if (gp != NULL) {
			if ((group = getgrnam(gp)) == NULL) {
				cerr << "Error: getgrnam: unknown group `" <<
					gp << "'" << endl;
				error = 1;
				goto shutdown_label;
			}
			gid = group->gr_gid;
		} else
			gid = passwd->pw_gid;

		if (!insane) {
			if (chroot(cfg->getChrootDir()) == -1) {
				cerr << "Error: chroot(\"" <<
					cfg->getChrootDir() << "\"): " <<
					strerror(errno) << endl;
				error = 1;
				goto shutdown_label;
			}
			if (chdir("/") == -1) {
				cerr << "Error: chdir(\"/\"): " <<
					strerror(errno) << endl;
				error = 1;
				goto shutdown_label;
			}
		}

#if defined(HAVE_SETRESGID) && defined(HAVE_SETRESUID)
		if (setgroups(1, &gid) ||
		    setresgid(gid, gid, gid) ||
		    setresuid(passwd->pw_uid, passwd->pw_uid, passwd->pw_uid))
#else
		if (setgroups(1, &gid) ||
		    setregid(gid, gid) ||
		    setreuid(passwd->pw_uid, passwd->pw_uid))
#endif
		{
			cerr << "Error: error while dropping privileges: " <<
				strerror(errno) << endl;
			error = 1;
			goto shutdown_label;
		}
	} else if (cfg->userRequestsPrivdrop()) {
		cerr << "Error: need root to drop privileges to a specific "
			"user or group." << endl;
		error = 1;
		goto shutdown_label;
	}

	/*
	 * Set up signals:
	 */
	memset(&act, 0, sizeof(act));
	act.sa_handler = sighdlr;
	act.sa_flags = SA_RESTART;
	sigemptyset(&act.sa_mask);
	SIGS_INSTALL(xsi_signals, &act);
	/* Ignore SIGPIPE */
	act.sa_handler = SIG_IGN;
	sigaction(SIGPIPE, &act, NULL);
	/* Handle signals that crash XMLSysInfo  */
	act.sa_handler = crashhdlr;
	SIGS_INSTALL(xsi_crash_sigs, &act);

	/*
	 * Become a daemon unless requested otherwise:
	 */
	if (!cfg->dontDetach())
		xsi_daemonize();

	/*
	 * Set the update timer:
	 */
	if (!cfg->oneClientMode()) {
		memset(&updateTimer, 0, sizeof(updateTimer));
		updateTimer.it_value.tv_sec = (long)cfg->getUpdateInterval();
		updateTimer.it_interval.tv_sec = (long)cfg->getUpdateInterval();
		if (setitimer(ITIMER_REAL, &updateTimer, NULL) == -1) {
			cerr << "Error: setitimer: " << strerror(errno)
				<< endl;
			error = 1;
			goto shutdown_label;
		}
	}

	/*
	 * We're ready to roll:
	 */
	XSI_Log::log()->notice("%s (%s) version %s ready",
			       PACKAGE_FULL_NAME, XSI_PLATFORM,
			       PACKAGE_VERSION);
	if (cfg->oneClientMode())
		XSI_Log::log()->notice("[config] One-Client Mode");
	else
		XSI_Log::log()->notice("[config] Standard mode: "
				       "update interval %d seconds",
				       cfg->getUpdateInterval());

	/* Collect static information: */
	sysinfo->readSysInfo();

	/*
	 * Main poll loop:
	 */
	while (!quit) {
		sigset_t signal_set;

		clients = server->poll();

		SIGS_SUSPEND(xsi_signals, &signal_set);

		if (s_error) {
			XSI_Log::log()->warn(__func__,
					     "Internal error: Signal handler called for unknown signal %d",
					     s_error);
			error = 1;
		} else if (clients.size() > 0) {
			vector<struct client>::iterator it;
			int format = 0, len;

			/* Update now if we're in One-Client Mode: */
			if (cfg->oneClientMode())
				sysinfo->updateSysInfo();

			if (cfg->formatOutput() && !cfg->compressXml())
				format = 1;

			/* Build and get the XML document: */
			sysinfo->buildSysInfoDoc();
			if ((xmlDocument = sysinfo->getSysInfoDocPtr()) == NULL) {
				XSI_Log::log()->warn(__func__,
						     "Internal error: xmlDocument is NULL");
				error = 1;
				break;
			} else
				/* Create a char string from the XML tree: */
				xmlDocDumpFormatMemoryEnc(xmlDocument, &buffer,
							  &len, "UTF-8",
							  format);

			/*
			 * Iterate through all clients that requested data
			 * this time:
			 */
			for (it = clients.begin(); it != clients.end(); it++) {
				ssize_t sent = 0;
				int	ni_error;
				char	hbuf[MAXHOSTNAMELEN], sbuf[NI_MAXSERV];

				if (error)
					/*
					 * We entered the loop with an error
					 * condition ...
					 */
					break;

				/* Find out who we're talking to: */
				ni_error = getnameinfo((struct sockaddr *)&it->c_addr,
						       it->c_addr_len, hbuf, sizeof(hbuf),
						       sbuf, sizeof(sbuf),
						       NI_NUMERICHOST | NI_NUMERICSERV);
				if (ni_error) {
					XSI_Log::log()->warn(__func__,
							     "getnameinfo: %s",
							     gai_strerror(ni_error));
					error = 1;
					break;
				}

				/* Send data (compressed or uncompressed): */
				if (cfg->compressXml()) {
					unsigned long zLen = 0;
					ssize_t bytes;

					if ((zBuffer = zCompress(buffer, (size_t)len, &zLen)) == NULL) {
						error = 1;
						break;
					}
					if ((bytes = write(it->fd, zBuffer, (size_t)zLen)) == -1)
						XSI_Log::log()->warn(NULL, errno,
								     "[write] while serving [%s]:%s",
								     hbuf, sbuf);
					else
						sent += bytes;
				} else {
					ssize_t bytes;

					if ((bytes = write(it->fd, buffer, (size_t)len)) == -1)
						XSI_Log::log()->warn(NULL, errno,
								     "[write] while serving [%s]:%s",
								     hbuf, sbuf);
					else
						sent += bytes;
				}

				close(it->fd);
				request_counter++;

				if (sent > 0)
					XSI_Log::log()->notice("[write] sent %ld bytes to [%s]:%s",
					      (long int)sent, hbuf, sbuf);

				xmlFree(buffer);
				buffer = NULL;
				zFree(zBuffer);
			}
		}

		if (reread) {
			XSI_Log::log()->warn(NULL, "[update] rereading system information, resetting cache(s)");
			sysinfo->resetCaches();
			sysinfo->readSysInfo();
			reread = 0;
		} else if (update) {
			sysinfo->updateSysInfo();
			update = 0;
		}

		if (pstats) {
			struct timeval	cur_time;
			uint64_t	runtime_secs;
			uint32_t	days, hours, minutes;
#ifdef HAVE_GETRUSAGE
			struct rusage	rusage;
#endif

			XSI_Log::log()->warn(NULL, "[stats]   requests:  %" PRIu64,
					     request_counter);

			if (gettimeofday(&cur_time, NULL) == -1)
				XSI_Log::log()->err(__func__, errno, "gettimeofday");
			runtime_secs = cur_time.tv_sec - xsi_starttime.tv_sec;
			days = runtime_secs / 86400;
			runtime_secs %= 86400;
			hours = runtime_secs / 3600;
			runtime_secs %= 3600;
			minutes = runtime_secs / 60;
			runtime_secs %= 60;
			XSI_Log::log()->warn(NULL, "[stats]    runtime:  "
					     "%" PRIu32 "d %02" PRIu32 "h %02"
					     PRIu32 "m %02" PRIu64 "s",
					     days, hours, minutes, runtime_secs);

#ifdef HAVE_GETRUSAGE
			if (getrusage(RUSAGE_SELF, &rusage) == 0) {
				double	utime, stime;

				utime = (double)rusage.ru_utime.tv_sec;
				utime += (double)rusage.ru_utime.tv_usec / 1000000.0;
				stime = (double)rusage.ru_stime.tv_sec;
				stime += (double)rusage.ru_stime.tv_usec / 1000000.0;
				XSI_Log::log()->warn(NULL, "[stats]     rusage:  %.3fs usr  %.3fs sys",
						     utime, stime);
			} else
				XSI_Log::log()->err(__func__, errno, "getrusage");
#endif

			XSI_Log::log()->warn(NULL, "[stats] user_cache:  %s",
					     cfg->noNames() ? "(unused)" :
					     sysinfo->getUserCacheStatsString());

			pstats = 0;
		}

		if (error) {
			XSI_Log::log()->warn(NULL, "[shutdown] Shutting down due to error(s)");
			quit = 1;
		}

		if (quit)
			XSI_Log::log()->warn(NULL, "[shutdown] %" PRIu64 " customer%s served",
					     request_counter,
					     request_counter == 1 ? "" : "s");

		SIGS_RESTORE(&signal_set);
	}

shutdown_label:
	if (sysinfo != NULL)
		delete sysinfo;
	if (server != NULL)
		delete server;
	if (cfg != NULL)
		delete cfg;
	delete[] __progname;

	return (error);
}
