/*	$Kolabore: network.cpp,v 1.15 2007/01/03 12:19:33 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "log.h"

extern "C" {
#include <sys/param.h>
#ifdef HAVE_LINUX_PARAM_H
#  include <linux/param.h>
#endif

#ifndef MAXHOSTNAMELEN
#  include <netdb.h>
#  ifndef MAXHOSTNAMELEN
#    define MAXHOSTNAMELEN 256
#  endif
#endif
#include <unistd.h>
}

#include <string>
#include <vector>

#include <cerrno>

#include "network.h"

using namespace std;

void
XSI_Network::setNetIfStats(vector<XSI_NetIfStat> nstats)
{
	net_stats = nstats;
}

void
XSI_Network::setDefaultRoutes(vector<struct default_route> routes)
{
	default_routes = routes;
}

const char *
XSI_Network::getHostName(void)
{
	char buf[MAXHOSTNAMELEN];

	if (gethostname(buf, sizeof(buf)) == -1)
		XSI_Log::log()->err(__func__, errno, "gethostname");
	hostname = buf;

	sanitizeString(hostname);
	return (hostname.c_str());
}

vector<XSI_NetIfStat>
XSI_Network::getNetIfStats(void)
{
	return (net_stats);
}

vector<struct default_route>
XSI_Network::getDefaultRoutes(void)
{
	return (default_routes);
}
