/*	$Kolabore: packetfilter.cpp,v 1.40 2007/02/09 13:04:18 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "log.h"

extern "C" {
#include <sys/types.h>
}

#include <string>
#include <vector>

#include <cstdlib>
#include <cstring>
#include <ctime>

#include "packetfilter.h"
#include "timekeeper.h"
#include "xmlsysinfo.h"

#define SMALLBUFSIZ	64

using namespace std;

FilteredIfStat::FilteredIfStat(const char *name)
{
	if (name == NULL)
		is_summary = true;
	else {
		is_summary = false;
		ifname = name;
	}

	ipv4_is_summary = false;

	pkts_v4_out_blocked = 0;
	pkts_v4_out_passed = 0;
	pkts_v4_in_blocked = 0;
	pkts_v4_in_passed = 0;
	pkts_v6_out_blocked = 0;
	pkts_v6_out_passed = 0;
	pkts_v6_in_blocked = 0;
	pkts_v6_in_passed = 0;
	bytes_v4_out_blocked = 0;
	bytes_v4_out_passed = 0;
	bytes_v4_in_blocked = 0;
	bytes_v4_in_passed = 0;
	bytes_v6_out_blocked = 0;
	bytes_v6_out_passed = 0;
	bytes_v6_in_blocked = 0;
	bytes_v6_in_passed = 0;
	last_pkts_v4_out_blocked = 0;
	last_pkts_v4_out_passed = 0;
	last_pkts_v4_in_blocked = 0;
	last_pkts_v4_in_passed = 0;
	last_pkts_v6_out_blocked = 0;
	last_pkts_v6_out_passed = 0;
	last_pkts_v6_in_blocked = 0;
	last_pkts_v6_in_passed = 0;
	last_bytes_v4_out_blocked = 0;
	last_bytes_v4_out_passed = 0;
	last_bytes_v4_in_blocked = 0;
	last_bytes_v4_in_passed = 0;
	last_bytes_v6_out_blocked = 0;
	last_bytes_v6_out_passed = 0;
	last_bytes_v6_in_blocked = 0;
	last_bytes_v6_in_passed = 0;
}

void
FilteredIfStat::setIPv4IsSummaryTraffic(bool ipv4sumsup)
{
	ipv4_is_summary = ipv4sumsup;
}

void
FilteredIfStat::setPktsV4OutBlocked(uint64_t n)
{
	last_pkts_v4_out_blocked = pkts_v4_out_blocked;
	pkts_v4_out_blocked = n;
}

void
FilteredIfStat::setPktsV4OutPassed(uint64_t n )
{
	last_pkts_v4_out_passed = pkts_v4_out_passed;
	pkts_v4_out_passed = n;
}

void
FilteredIfStat::setPktsV4InBlocked(uint64_t n)
{
	last_pkts_v4_in_blocked = pkts_v4_in_blocked;
	pkts_v4_in_blocked = n;
}

void
FilteredIfStat::setPktsV4InPassed(uint64_t n)
{
	last_pkts_v4_in_passed = pkts_v4_in_passed;
	pkts_v4_in_passed = n;
}

void
FilteredIfStat::setPktsV6OutBlocked(uint64_t n)
{
	last_pkts_v6_out_blocked = pkts_v6_out_blocked;
	pkts_v6_out_blocked = n;
}

void
FilteredIfStat::setPktsV6OutPassed(uint64_t n)
{
	last_pkts_v6_out_passed = pkts_v6_out_passed;
	pkts_v6_out_passed = n;
}

void
FilteredIfStat::setPktsV6InBlocked(uint64_t n)
{
	last_pkts_v6_in_blocked = pkts_v6_in_blocked;
	pkts_v6_in_blocked = n;
}

void
FilteredIfStat::setPktsV6InPassed(uint64_t n)
{
	last_pkts_v6_in_passed = pkts_v6_in_passed;
	pkts_v6_in_passed = n;
}

void
FilteredIfStat::setBytesV4OutBlocked(uint64_t n)
{
	last_bytes_v4_out_blocked = bytes_v4_out_blocked;
	bytes_v4_out_blocked = n;
}

void
FilteredIfStat::setBytesV4OutPassed(uint64_t n)
{
	last_bytes_v4_out_passed = bytes_v4_out_passed;
	bytes_v4_out_passed = n;
}

void
FilteredIfStat::setBytesV4InBlocked(uint64_t n)
{
	last_bytes_v4_in_blocked = bytes_v4_in_blocked;
	bytes_v4_in_blocked = n;
}

void
FilteredIfStat::setBytesV4InPassed(uint64_t n)
{
	last_bytes_v4_in_passed = bytes_v4_in_passed;
	bytes_v4_in_passed = n;
}

void
FilteredIfStat::setBytesV6OutBlocked(uint64_t n)
{
	last_bytes_v6_out_blocked = bytes_v6_out_blocked;
	bytes_v6_out_blocked = n;
}

void
FilteredIfStat::setBytesV6OutPassed(uint64_t n)
{
	last_bytes_v6_out_passed = bytes_v6_out_passed;
	bytes_v6_out_passed = n;
}

void
FilteredIfStat::setBytesV6InBlocked(uint64_t n)
{
	last_bytes_v6_in_blocked = bytes_v6_in_blocked;
	bytes_v6_in_blocked = n;
}

void
FilteredIfStat::setBytesV6InPassed(uint64_t n)
{
	last_bytes_v6_in_passed = bytes_v6_in_passed;
	bytes_v6_in_passed = n;
}

bool
FilteredIfStat::isSummary(void)
{
	return (is_summary);
}

bool
FilteredIfStat::v4CountersReturnSummaryTraffic(void)
{
	return (ipv4_is_summary);
}

bool
FilteredIfStat::hasIPv4Traffic(void)
{
	return (pkts_v4_out_blocked > 0 ||
		pkts_v4_out_passed > 0 ||
		pkts_v4_in_blocked > 0 ||
		pkts_v4_in_passed > 0 ||
		bytes_v4_out_blocked > 0 ||
		bytes_v4_out_passed > 0 ||
		bytes_v4_in_blocked > 0 ||
		bytes_v4_in_passed > 0);
}

bool
FilteredIfStat::hasIPv6Traffic(void)
{
	return (pkts_v6_out_blocked > 0 ||
		pkts_v6_out_passed > 0 ||
		pkts_v6_in_blocked > 0 ||
		pkts_v6_in_passed > 0 ||
		bytes_v6_out_blocked > 0 ||
		bytes_v6_out_passed > 0 ||
		bytes_v6_in_blocked > 0 ||
		bytes_v6_in_passed > 0);
}

bool
FilteredIfStat::hasTraffic(void)
{
	return (hasIPv4Traffic() || hasIPv6Traffic());
}

bool
FilteredIfStat::hasByteCounts(void)
{
	return (bytes_v4_out_blocked > 0 ||
		bytes_v4_out_passed > 0 ||
		bytes_v4_in_blocked > 0 ||
		bytes_v4_in_passed > 0 ||
		bytes_v6_out_blocked > 0 ||
		bytes_v6_out_passed > 0 ||
		bytes_v6_in_blocked > 0 ||
		bytes_v6_in_passed > 0);
}

const char *
FilteredIfStat::getIfName(bool sanitize)
{
	if (is_summary)
		return (NULL);
	else {
		if (sanitize)
			sanitizeString(ifname);
		return (ifname.c_str());
	}
}

const char *
FilteredIfStat::getPktsV4OutBlockedString(void)
{
	return (makeIntStr(pkts_v4_out_blocked));
}

const char *
FilteredIfStat::getPktsV4OutPassedString(void)
{
	return (makeIntStr(pkts_v4_out_passed));
}

const char *
FilteredIfStat::getPktsV4InBlockedString(void)
{
	return (makeIntStr(pkts_v4_in_blocked));
}

const char *
FilteredIfStat::getPktsV4InPassedString(void)
{
	return (makeIntStr(pkts_v4_in_passed));
}

const char *
FilteredIfStat::getPktsV6OutBlockedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(pkts_v6_out_blocked));
}

const char *
FilteredIfStat::getPktsV6OutPassedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(pkts_v6_out_passed));
}

const char *
FilteredIfStat::getPktsV6InBlockedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(pkts_v6_in_blocked));
}

const char *
FilteredIfStat::getPktsV6InPassedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(pkts_v6_in_passed));
}

const char *
FilteredIfStat::getBytesV4OutBlockedString(void)
{
	return (makeIntStr(bytes_v4_out_blocked));
}

const char *
FilteredIfStat::getBytesV4OutPassedString(void)
{
	return (makeIntStr(bytes_v4_out_passed));
}

const char *
FilteredIfStat::getBytesV4InBlockedString(void)
{
	return (makeIntStr(bytes_v4_in_blocked));
}

const char *
FilteredIfStat::getBytesV4InPassedString(void)
{
	return (makeIntStr(bytes_v4_in_passed));
}

const char *
FilteredIfStat::getBytesV6OutBlockedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(bytes_v6_out_blocked));
}

const char *
FilteredIfStat::getBytesV6OutPassedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(bytes_v6_out_passed));
}

const char *
FilteredIfStat::getBytesV6InBlockedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(bytes_v6_in_blocked));
}

const char *
FilteredIfStat::getBytesV6InPassedString(void)
{
	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	return (makeIntStr(bytes_v6_in_passed));
}

const char *
FilteredIfStat::getPktsV4OutBlockedPerSecString(void)
{
	uint64_t value;

	if (!pkts_v4_out_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v4_out_blocked)
		return (NULL);

	value = pkts_v4_out_blocked - last_pkts_v4_out_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV4OutPassedPerSecString(void)
{
	uint64_t value;

	if (!pkts_v4_out_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v4_out_passed)
		return (NULL);

	value = pkts_v4_out_passed - last_pkts_v4_out_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV4InBlockedPerSecString(void)
{
	uint64_t value;

	if (!pkts_v4_in_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v4_in_blocked)
		return (NULL);

	value = pkts_v4_in_blocked - last_pkts_v4_in_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV4InPassedPerSecString(void)
{
	uint64_t value;

	if (!pkts_v4_in_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v4_in_passed)
		return (NULL);

	value = pkts_v4_in_passed - last_pkts_v4_in_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV6OutBlockedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (pkts_v6_out_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v6_out_blocked)
		return (NULL);

	value = pkts_v6_out_blocked - last_pkts_v6_out_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV6OutPassedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (pkts_v6_out_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v6_out_passed)
		return (NULL);

	value = pkts_v6_out_passed - last_pkts_v6_out_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV6InBlockedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (!pkts_v6_in_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v6_in_blocked)
		return (NULL);

	value = pkts_v6_in_blocked - last_pkts_v6_in_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getPktsV6InPassedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (!pkts_v6_in_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_pkts_v6_in_passed)
		return (NULL);

	value = pkts_v6_in_passed - last_pkts_v6_in_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV4OutBlockedPerSecString(void)
{
	uint64_t value;

	if (!bytes_v4_out_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v4_out_blocked)
		return (NULL);

	value = bytes_v4_out_blocked - last_bytes_v4_out_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV4OutPassedPerSecString(void)
{
	uint64_t value;

	if (!bytes_v4_out_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v4_out_passed)
		return (NULL);

	value = bytes_v4_out_passed - last_bytes_v4_out_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV4InBlockedPerSecString(void)
{
	uint64_t value;

	if (!bytes_v4_in_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v4_in_blocked)
		return (NULL);

	value = bytes_v4_in_blocked - last_bytes_v4_in_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV4InPassedPerSecString(void)
{
	uint64_t value;

	if (!bytes_v4_in_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v4_in_passed)
		return (NULL);

	value = bytes_v4_in_passed - last_bytes_v4_in_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV6OutBlockedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (!bytes_v6_out_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v6_out_blocked)
		return (NULL);

	value = bytes_v6_out_blocked - last_bytes_v6_out_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV6OutPassedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (!bytes_v6_out_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v6_out_passed)
		return (NULL);

	value = bytes_v6_out_passed - last_bytes_v6_out_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV6InBlockedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (!bytes_v6_in_blocked || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v6_in_blocked)
		return (NULL);

	value = bytes_v6_in_blocked - last_bytes_v6_in_blocked;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}

const char *
FilteredIfStat::getBytesV6InPassedPerSecString(void)
{
	uint64_t value;

	if (ipv4_is_summary)
		XSI_Log::log()->err(__func__, "Internal error: IPv6-specific stats queried from summary-only FilteredIfStat object");

	if (!bytes_v6_in_passed || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_bytes_v6_in_passed)
		return (NULL);

	value = bytes_v6_in_passed - last_bytes_v6_in_passed;

	if (value == 0)
		return (NULL);

	return (makeFloatStr(value, XSI_TimeKeeper::get()->timeDelta()));
}


XSI_PFilter::XSI_PFilter(void)
{
	is_available = false;
	type = PFILTER_UNSPEC;
	enabled = false;
	have_states = false;
	runtime = 0;
	rule_matches = last_rule_matches = 0;
	states = 0;
	states_limit = -1;
	state_searches = last_state_searches = 0;
	state_inserts = last_state_inserts = 0;
	state_removals = last_state_removals = 0;
	state_mismatches = last_state_mismatches = 0;
}

void
XSI_PFilter::setAvailable(bool available)
{
	is_available = available;
}

void
XSI_PFilter::setPFilterType(enum xsi_pfilter_type t)
{
	type = t;
}

void
XSI_PFilter::setEnabled(bool e)
{
	enabled = e;
}

void
XSI_PFilter::setRuntime(time_t rtime)
{
	runtime = rtime;
}

void
XSI_PFilter::setRuleMatches(uint64_t matches)
{
	last_rule_matches = rule_matches;
	rule_matches = matches;
}

void
XSI_PFilter::setStates(uint32_t s)
{
	if (s > 0)
		have_states = true;
	states = s;
}

void
XSI_PFilter::setStatesLimit(int64_t l)
{
	states_limit = l;
}

void
XSI_PFilter::setStateSearches(uint64_t searches)
{
	if (searches > 0)
		have_states = true;
	last_state_searches = state_searches;
	state_searches = searches;
}

void
XSI_PFilter::setStateInserts(uint64_t inserts)
{
	last_state_inserts = state_inserts;
	state_inserts = inserts;
}

void
XSI_PFilter::setStateRemovals(uint64_t removals)
{
	last_state_removals = state_removals;
	state_removals = removals;
}

void
XSI_PFilter::setStateMismatches(uint64_t mismatches)
{
	last_state_mismatches = state_mismatches;
	state_mismatches = mismatches;
}

void
XSI_PFilter::setFilteredIfStats(vector<FilteredIfStat> fifs)
{
	filteredIfStats = fifs;
}

bool
XSI_PFilter::isAvailable(void)
{
	return (is_available);
}

const char *
XSI_PFilter::getPFilterTypeString(void)
{
	switch (type) {
	case PFILTER_UNSPEC:
		XSI_Log::log()->err(__func__, "Internal error: Packet filter class neglected to set filter type");
	case PFILTER_PF:
		return ("pf");
	case PFILTER_IPFILTER:
		return ("ipfilter");
	case PFILTER_NETFILTER:
		return ("netfilter");
	default:
		XSI_Log::log()->err(__func__, "Internal error: Unknown packet filter set");
	}

	/* NOTREACHED */
	/* Squelch warnings: */
	exit(1);
}

bool
XSI_PFilter::isEnabled(void)
{
	return (enabled);
}

bool
XSI_PFilter::hasStates(void)
{
	return (have_states);
}

const char *
XSI_PFilter::getRuntimeSecString(void)
{
	if (runtime > 0)
		return (makeIntStr((int64_t)runtime));
	else
		return (NULL);
}

const char *
XSI_PFilter::getRuleMatchesString(void)
{
	if (rule_matches > 0)
		return (makeIntStr(rule_matches));
	else
		return (NULL);
}

const char *
XSI_PFilter::getRuleMatchesAverageString(void)
{
	if (runtime > 0)
		return (makeFloatStr(rule_matches, (double)runtime));
	else
		return (NULL);
}

const char *
XSI_PFilter::getRuleMatchesPerSecString(void)
{
	if (!rule_matches || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_rule_matches || rule_matches < last_rule_matches
	    || rule_matches - last_rule_matches == 0)
		return (NULL);
	else
		return (makeFloatStr(rule_matches - last_rule_matches,
				     XSI_TimeKeeper::get()->timeDelta()));
}

const char *
XSI_PFilter::getStatesString(void)
{
	return (makeIntStr(states));
}

const char *
XSI_PFilter::getStatesLimitString(void)
{
	if (states_limit < 0)
		return (NULL);
	else
		return (makeIntStr(states_limit));
}

const char *
XSI_PFilter::getStateSearchesString(void)
{
	if (state_searches > 0)
		return (makeIntStr(state_searches));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateSearchesAverageString(void)
{
	if (runtime > 0)
		return (makeFloatStr(state_searches, (double)runtime));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateSearchesPerSecString(void)
{
	if (!state_searches || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_state_searches || state_searches < last_state_searches
	    || state_searches - last_state_searches == 0)
		return (NULL);
	else
		return (makeFloatStr(state_searches - last_state_searches,
				     XSI_TimeKeeper::get()->timeDelta()));
}

const char *
XSI_PFilter::getStateInsertsString(void)
{
	if (state_inserts > 0)
		return (makeIntStr(state_inserts));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateInsertsAverageString(void)
{
	if (runtime > 0)
		return (makeFloatStr(state_inserts, (double)runtime));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateInsertsPerSecString(void)
{
	if (!state_inserts || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_state_inserts || state_inserts < last_state_inserts
	    || state_inserts - last_state_inserts == 0)
		return (NULL);
	else
		return (makeFloatStr(state_inserts - last_state_inserts,
				     XSI_TimeKeeper::get()->timeDelta()));
}

const char *
XSI_PFilter::getStateRemovalsString(void)
{
	if (state_removals > 0)
		return (makeIntStr(state_removals));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateRemovalsAverageString(void)
{
	if (runtime > 0)
		return (makeFloatStr(state_removals, (double)runtime));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateRemovalsPerSecString(void)
{
	if (!state_removals || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_state_removals || state_removals < last_state_removals
	    || state_removals - last_state_removals == 0)
		return (NULL);
	else
		return (makeFloatStr(state_removals - last_state_removals,
				     XSI_TimeKeeper::get()->timeDelta()));
}

const char *
XSI_PFilter::getStateMismatchesString(void)
{
	if (state_mismatches > 0)
		return (makeIntStr(state_mismatches));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateMismatchesAverageString(void)
{
	if (runtime > 0)
		return (makeFloatStr(state_mismatches, (double)runtime));
	else
		return (NULL);
}

const char *
XSI_PFilter::getStateMismatchesPerSecString(void)
{
	if (!state_mismatches || XSI_TimeKeeper::get()->timeDelta() < 0.01
	    || !last_state_mismatches || state_mismatches < last_state_mismatches
	    || state_mismatches - last_state_mismatches == 0)
		return (NULL);
	else
		return (makeFloatStr(state_mismatches - last_state_mismatches,
				     XSI_TimeKeeper::get()->timeDelta()));
}

vector<FilteredIfStat>
XSI_PFilter::getFilteredIfStats(void)
{
	return (filteredIfStats);
}
