/*	$Kolabore: packetfilter.h,v 1.26 2007/01/03 12:19:33 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * packetfilter.h
 * Interface to packet filter (PF, netfilter, ...) statistics
 */

#ifndef __PACKETFILTER_H__
#define __PACKETFILTER_H__

#include "util.h"

#define XSI_PFILTER_PF		 0
#define XSI_PFILTER_IPFILTER	 1
#define XSI_PFILTER_NETFILTER	 2
#define XSI_PFILTER_MAX 	 3

using namespace std;

enum xsi_pfilter_type {
	PFILTER_PF = 0,
	PFILTER_IPFILTER,
	PFILTER_NETFILTER,
	PFILTER_UNSPEC
};

class FilteredIfStat : private XSI_Util {
public:
	/*
	 * For packet filters that suck, with no interface-specific counters:
	 * Provide only one single FilteredIfStat object, and construct it with
	 * a NULL pointer. This will turn it into the <filter_summary/>
	 * element's content, instead of many interesting <filtered_if/>
	 * elements. Avoid if possible.
	 */
	FilteredIfStat(const char * /* interface name or NULL */);

	void		setIPv4IsSummaryTraffic(bool);
	void		setPktsV4OutBlocked(uint64_t);
	void		setPktsV4OutPassed(uint64_t);
	void		setPktsV4InBlocked(uint64_t);
	void		setPktsV4InPassed(uint64_t);
	void		setPktsV6OutBlocked(uint64_t);
	void		setPktsV6OutPassed(uint64_t);
	void		setPktsV6InBlocked(uint64_t);
	void		setPktsV6InPassed(uint64_t);
	void		setBytesV4OutBlocked(uint64_t);
	void		setBytesV4OutPassed(uint64_t);
	void		setBytesV4InBlocked(uint64_t);
	void		setBytesV4InPassed(uint64_t);
	void		setBytesV6OutBlocked(uint64_t);
	void		setBytesV6OutPassed(uint64_t);
	void		setBytesV6InBlocked(uint64_t);
	void		setBytesV6InPassed(uint64_t);

	bool		isSummary(void);
	bool		v4CountersReturnSummaryTraffic(void);
	bool		hasIPv4Traffic(void);
	bool		hasIPv6Traffic(void);
	bool		hasTraffic(void);
	bool		hasByteCounts(void);
	const char *	getIfName(bool sanitize = false);
	const char *	getPktsV4OutBlockedString(void);
	const char *	getPktsV4OutPassedString(void);
	const char *	getPktsV4InBlockedString(void);
	const char *	getPktsV4InPassedString(void);
	const char *	getPktsV6OutBlockedString(void);
	const char *	getPktsV6OutPassedString(void);
	const char *	getPktsV6InBlockedString(void);
	const char *	getPktsV6InPassedString(void);
	const char *	getBytesV4OutBlockedString(void);
	const char *	getBytesV4OutPassedString(void);
	const char *	getBytesV4InBlockedString(void);
	const char *	getBytesV4InPassedString(void);
	const char *	getBytesV6OutBlockedString(void);
	const char *	getBytesV6OutPassedString(void);
	const char *	getBytesV6InBlockedString(void);
	const char *	getBytesV6InPassedString(void);
	const char *	getPktsV4OutBlockedPerSecString(void);
	const char *	getPktsV4OutPassedPerSecString(void);
	const char *	getPktsV4InBlockedPerSecString(void);
	const char *	getPktsV4InPassedPerSecString(void);
	const char *	getPktsV6OutBlockedPerSecString(void);
	const char *	getPktsV6OutPassedPerSecString(void);
	const char *	getPktsV6InBlockedPerSecString(void);
	const char *	getPktsV6InPassedPerSecString(void);
	const char *	getBytesV4OutBlockedPerSecString(void);
	const char *	getBytesV4OutPassedPerSecString(void);
	const char *	getBytesV4InBlockedPerSecString(void);
	const char *	getBytesV4InPassedPerSecString(void);
	const char *	getBytesV6OutBlockedPerSecString(void);
	const char *	getBytesV6OutPassedPerSecString(void);
	const char *	getBytesV6InBlockedPerSecString(void);
	const char *	getBytesV6InPassedPerSecString(void);

private:
	bool		is_summary, ipv4_is_summary;
	string		ifname;
	uint64_t	pkts_v4_out_blocked,	last_pkts_v4_out_blocked;
	uint64_t	pkts_v4_out_passed,	last_pkts_v4_out_passed;
	uint64_t	pkts_v4_in_blocked,	last_pkts_v4_in_blocked;
	uint64_t	pkts_v4_in_passed,	last_pkts_v4_in_passed;
	uint64_t	pkts_v6_out_blocked,	last_pkts_v6_out_blocked;
	uint64_t	pkts_v6_out_passed,	last_pkts_v6_out_passed;
	uint64_t	pkts_v6_in_blocked,	last_pkts_v6_in_blocked;
	uint64_t	pkts_v6_in_passed,	last_pkts_v6_in_passed;
	uint64_t	bytes_v4_out_blocked,	last_bytes_v4_out_blocked;
	uint64_t	bytes_v4_out_passed,	last_bytes_v4_out_passed;
	uint64_t	bytes_v4_in_blocked,	last_bytes_v4_in_blocked;
	uint64_t	bytes_v4_in_passed,	last_bytes_v4_in_passed;
	uint64_t	bytes_v6_out_blocked,	last_bytes_v6_out_blocked;
	uint64_t	bytes_v6_out_passed,	last_bytes_v6_out_passed;
	uint64_t	bytes_v6_in_blocked,	last_bytes_v6_in_blocked;
	uint64_t	bytes_v6_in_passed,	last_bytes_v6_in_passed;
};

class XSI_PFilter : private XSI_Util {
public:
	XSI_PFilter(void);

	void		setAvailable(bool);
	void		setPFilterType(enum xsi_pfilter_type);
	void		setEnabled(bool);
	void		setRuntime(time_t);
	void		setRuleMatches(uint64_t);
	void		setStates(uint32_t);
	void		setStatesLimit(int64_t);
	void		setStateSearches(uint64_t);
	void		setStateInserts(uint64_t);
	void		setStateRemovals(uint64_t);
	void		setStateMismatches(uint64_t);
	void		setFilteredIfStats(vector<FilteredIfStat>);

	bool		isAvailable(void);
	const char *	getPFilterTypeString(void);
	bool		isEnabled(void);
	bool		hasStates(void);
	const char *	getRuntimeSecString(void);
	const char *	getRuleMatchesString(void);
	const char *	getRuleMatchesAverageString(void);
	const char *	getRuleMatchesPerSecString(void);
	const char *	getStatesString(void);
	const char *	getStatesLimitString(void);
	const char *	getStateSearchesString(void);
	const char *	getStateSearchesAverageString(void);
	const char *	getStateSearchesPerSecString(void);
	const char *	getStateInsertsString(void);
	const char *	getStateInsertsAverageString(void);
	const char *	getStateInsertsPerSecString(void);
	const char *	getStateRemovalsString(void);
	const char *	getStateRemovalsAverageString(void);
	const char *	getStateRemovalsPerSecString(void);
	const char *	getStateMismatchesString(void);
	const char *	getStateMismatchesAverageString(void);
	const char *	getStateMismatchesPerSecString(void);
	vector<FilteredIfStat> getFilteredIfStats(void);

private:
	bool		is_available;
	enum xsi_pfilter_type type;
	bool		enabled, have_states;
	time_t		runtime;
	uint64_t	rule_matches,		last_rule_matches;
	uint32_t	states;
	int64_t 	states_limit;
	uint64_t	state_searches, 	last_state_searches;
	uint64_t	state_inserts,		last_state_inserts;
	uint64_t	state_removals, 	last_state_removals;
	uint64_t	state_mismatches,	last_state_mismatches;
	vector<FilteredIfStat> filteredIfStats;
};

#endif /* __PACKETFILTER_H__ */
