/*	$Kolabore: raid.h,v 1.17 2007/01/03 12:19:33 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * raid.h
 * Interface to SCSI RAID, of similar design as sensor support.
 */

#ifndef __RAID_H__
#define __RAID_H__

#include "util.h"

using namespace std;

enum xsi_raid_vol_status {
	XSI_RAID_VOL_ONLINE,
	XSI_RAID_VOL_OFFLINE,
	XSI_RAID_VOL_DEGRADED,
	XSI_RAID_VOL_BUILDING,
	XSI_RAID_VOL_SCRUBBING,
	XSI_RAID_VOL_REBUILD,
	XSI_RAID_VOL_INVALID,
	XSI_RAID_VOL_UNKNOWN,
	XSI_RAID_VOL_MAX
};

enum xsi_raid_disk_status {
	XSI_RAID_DSK_ONLINE,
	XSI_RAID_DSK_OFFLINE,
	XSI_RAID_DSK_FAILED,
	XSI_RAID_DSK_REBUILD,
	XSI_RAID_DSK_HOTSPARE,
	XSI_RAID_DSK_UNUSED,
	XSI_RAID_DSK_SCRUBBING,
	XSI_RAID_DSK_INVALID,
	XSI_RAID_DSK_UNKNOWN,
	XSI_RAID_DSK_MAX
};

class XSI_RDisk : private XSI_Util {
public:
	XSI_RDisk(void);

	void			setDiskID(int);
	void			setDiskLocation(int, int);
	void			setDiskLUN(int);
	void			setDiskStatus(enum xsi_raid_disk_status);
	void			setDiskSize(uint64_t);
	void			setDiskEnclosureDevName(const char *);
	void			setDiskVendorName(const char *);
	void			setDiskSerial(const char *);

	const char *		getDiskIDString(void);
	const char *		getDiskBusString(void);
	const char *		getDiskTargetString(void);
	const char *		getDiskLUNString(void);
	const char *		getDiskStatusString(void);
	const char *		getDiskSizeString(void);
	const char *		getDiskEnclosureDevName(bool sanitize = false);
	const char *		getDiskVendorName(bool sanitize = false);
	const char *		getDiskSerial(bool sanitize = false);

private:
	int			id;
	int			bus, target, lun;
	enum xsi_raid_disk_status status;
	uint64_t		size;
	string			enclosure;
	string			vendor;
	string			serial;
};

class XSI_RVolume : private XSI_Util {
public:
	XSI_RVolume(void);

	void			setVolumeDevName(const char *);
	void			setVolumeID(int);
	void			setRaidLevel(int);
	void			setVolumeStatus(enum xsi_raid_vol_status);
	void			setVolumeOpProgress(int);
	void			setVolumeOpElapsed(int);
	void			setVolumeSize(uint64_t);
	void			setVolumeDisks(vector<XSI_RDisk>);

	const char *		getVolumeDevName(void);
	const char *		getVolumeIDString(void);
	const char *		getRaidLevelString(void);
	const char *		getVolumeStatusString(void);
	const char *		getVolumeOpProgressString(void);
	const char *		getVolumeOpElapsedString(void);
	const char *		getVolumeSizeString(void);
	vector<XSI_RDisk>	getVolumeDisks(void);

private:
	string			device;
	int			id;
	int			level;
	enum xsi_raid_vol_status status;
	int			progress;
	int			elapsed;
	uint64_t		size;
	vector<XSI_RDisk>	disks;
};

class XSI_Raid : private XSI_Util {
public:
	XSI_Raid(const char * /* controller name */);

	void			setVolumes(vector<XSI_RVolume>);
	void			setReserveDisks(vector<XSI_RDisk>);

	vector<XSI_RVolume>	getVolumes(void);
	vector<XSI_RDisk>	getReserveDisks(void);
	const char *		getRaidControllerName(bool sanitize = false);

private:
	vector<XSI_RVolume>	volumes;
	vector<XSI_RDisk>	disks;
	string			controller;
};

#endif /* __RAID_H__ */
