/*	$Kolabore: storage.h,v 1.17 2007/01/05 21:27:45 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * storage.h
 * Disks, filesystems, etc.
 */

#ifndef __STORAGE_H__
#define __STORAGE_H__

#include "raid.h"
#include "util.h"

using namespace std;

/*
 * struct fs:
 * structure describing a filesystem
 */
struct fs {
	string		mountpoint;	// e.g. /usr
	string		fs_name;	// e.g. /dev/wd0d
	string		fs_type;	// e.g. ffs
	uint64_t	size_total;	// total space (bytes)
	uint64_t	size_free;	// free space (total)
	int64_t		size_avail;	// free space (actual, non-reserved
					//	      available space)
	uint64_t	nodes_total;	// file nodes / inodes
	uint64_t	nodes_free;
};

class XSI_DiskIO : private XSI_Util {
public:
	XSI_DiskIO(void);
	XSI_DiskIO(const char * /* disk name */);

	void		setDiskName(const char *);
	void		setWrittenBytes(uint64_t);
	void		setReadBytes(uint64_t);
	void		setWriteTransfers(uint64_t);
	void		setReadTransfers(uint64_t);

	bool		isIdle(void);
	bool		hasBPS(void);
	bool		hasTPS(void);
	const char *	getDiskName(bool sanitize = false);
	const char *	getWriteBPSString(void);
	const char *	getReadBPSString(void);
	const char *	getWriteTPSString(void);
	const char *	getReadTPSString(void);
	double		getWriteBPS(void);
	double		getReadBPS(void);
	double		getWriteTPS(void);
	double		getReadTPS(void);

private:
	string		disk_name;
	uint64_t	bytes_written, last_bytes_written;
	uint64_t	bytes_read, last_bytes_read;
	uint64_t	transfers_write, last_transfers_write;
	uint64_t	transfers_read, last_transfers_read;
};

/*
 * XSI_Storage class
 * Gets and outputs information about filesystems and disks
 *   MANDATORY: XSI needs to know about at least one filesystem
 *    OPTIONAL: Local disks, CD-ROMs, etc.
 */
class XSI_Storage : private XSI_Util {
public:
	XSI_Storage(void);

	void			setFilesystems(vector<struct fs>);
	void			setDisks(vector<XSI_DiskIO>);

	void			updateIOTotals(void);
	bool			hasFilesystemInfo(void);
	bool			hasDiskInfo(void);
	vector<struct fs>	getFilesystems(void);
	vector<XSI_DiskIO>	getDisks(void);
	const char *		getReadBPSTotalString(void);
	const char *		getWriteBPSTotalString(void);
	const char *		getReadTPSTotalString(void);
	const char *		getWriteTPSTotalString(void);

private:
	double			write_bps_total, read_bps_total;
	double			write_tps_total, read_tps_total;
	vector<struct fs>	filesystems;
	vector<XSI_DiskIO>	disks;
	bool			up2date;
};

#endif /* __STORAGE_H__ */
