/*	$Kolabore: timekeeper.cpp,v 1.15 2007/01/15 10:33:04 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "log.h"

extern "C" {
#include <sys/time.h>
#include <sys/types.h>
}

#include <cerrno>

#include "timekeeper.h"

using namespace std;

XSI_TimeKeeper *
XSI_TimeKeeper::get(void)
{
	static XSI_TimeKeeper instance;
	return (&instance);
}

bool
XSI_TimeKeeper::haveTimeDelta(void)
{
	return (have_time_delta);
}

double
XSI_TimeKeeper::timeDelta(void)
{
	return (update_time_delta);
}

const char *
XSI_TimeKeeper::updateTimestampString(void)
{
	return (makeFloatStr(timestamp, 1, 3));
}

uint32_t
XSI_TimeKeeper::updateTimestampInt(void)
{
	return ((uint32_t)(timestamp / 1.0));
}

XSI_TimeKeeper::XSI_TimeKeeper(void)
{
	have_time_delta = false;
	last_update_time.tv_sec = 0;
	last_update_time.tv_usec = 0;
	if (gettimeofday(&update_time, NULL) == -1)
		XSI_Log::log()->err(__func__, errno, "gettimeofday");
	update_time_delta = 1;
}

void
XSI_TimeKeeper::triggerUpdate(void)
{
	static bool firstrun = true;

	if (firstrun) {
		firstrun = false;
		return;
	}

	last_update_time.tv_sec = update_time.tv_sec;
	last_update_time.tv_usec = update_time.tv_usec;
	if (gettimeofday(&update_time, NULL) == -1)
		XSI_Log::log()->err(__func__, errno, "gettimeofday");
	update_time_delta =
		((double)update_time.tv_sec + (double)(update_time.tv_usec) / 1000000.0)
		- ((double)last_update_time.tv_sec + (double)(last_update_time.tv_usec) / 1000000.0);
	have_time_delta = true;
	timestamp = update_time.tv_sec;
	timestamp += (double)update_time.tv_usec / (double)1000000;
}
