/*	$Kolabore: util.cpp,v 1.31 2007/02/09 20:58:49 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "log.h"

#include <fstream>
#include <string>
#include <vector>

#include <cerrno>
#include <cstdio>

#include "util.h"

#define SMALLBUFSIZ	128

using namespace std;

const char *
XSI_Util::makeFloatStr(int64_t num, double divisor, unsigned short precision)
{
	char		buf[SMALLBUFSIZ];
	char		fmt[10];
	static string	ret;
	int		sret;

	sret = snprintf(fmt, sizeof(fmt), "%%.%huf", precision);
	if (sret < 0 || sret >= (int)sizeof(fmt))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");

	if (divisor < 0.0001)
		sret = snprintf(buf, sizeof(buf), fmt, 0.0);
        else
		sret = snprintf(buf, sizeof(buf), fmt, (double)num / divisor);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");

	ret = buf;
	return (ret.c_str());
}

const char *
XSI_Util::makeFloatStr(uint64_t num, double divisor, unsigned short precision)
{
	char		buf[SMALLBUFSIZ];
	char		fmt[10];
	static string	ret;
	int		sret;

	sret = snprintf(fmt, sizeof(fmt), "%%.%huf", precision);
	if (sret < 0 || sret >= (int)sizeof(fmt))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");

	if (divisor < 0.0001)
		sret = snprintf(buf, sizeof(buf), fmt, 0.0);
        else
		sret = snprintf(buf, sizeof(buf), fmt, (double)num / divisor);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");

	ret = buf;
	return (ret.c_str());
}

const char *
XSI_Util::makeFloatStr(double num, double divisor, unsigned short precision)
{
	char		buf[SMALLBUFSIZ];
	char		fmt[10];
	static string	ret;
	int		sret;

	sret = snprintf(fmt, sizeof(fmt), "%%.%huf", precision);
	if (sret < 0 || sret >= (int)sizeof(fmt))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");

	if (divisor < 0.0001)
		sret = snprintf(buf, sizeof(buf), fmt, 0.0);
        else
		sret = snprintf(buf, sizeof(buf), fmt, num / divisor);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");

	ret = buf;
	return (ret.c_str());
}

const char *
XSI_Util::makeIntStr(int64_t num)
{
	char		buf[SMALLBUFSIZ];
	static string	ret;
	int		sret;

	sret = snprintf(buf, sizeof(buf), "%" PRIi64, num);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");
	ret = buf;
	return (ret.c_str());
}

const char *
XSI_Util::makeIntStr(uint64_t num)
{
	char		buf[SMALLBUFSIZ];
	static string	ret;
	int		sret;

	sret = snprintf(buf, sizeof(buf), "%" PRIu64, num);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");
	ret = buf;
	return (ret.c_str());
}

const char *
XSI_Util::makeIntStr(int32_t num)
{
	char		buf[SMALLBUFSIZ];
	static string	ret;
	int		sret;

	sret = snprintf(buf, sizeof(buf), "%d", num);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");
	ret = buf;
	return (ret.c_str());
}

const char *
XSI_Util::makeIntStr(uint32_t num)
{
	char		buf[SMALLBUFSIZ];
	static string	ret;
	int		sret;

	sret = snprintf(buf, sizeof(buf), "%u", num);
	if (sret < 0 || sret >= (int)sizeof(buf))
		XSI_Log::log()->err(__func__, "snprintf: string truncation or format/encoding error");
	ret = buf;
	return (ret.c_str());
}

unsigned long
XSI_Util::sanitizeString(string &s, bool for_attribute)
{
	unsigned long		ret = 0;
	string::iterator	c;
	bool			appears_sanitized = false;

	if (s.empty())
		return (ret);

	/* Always get rid of non-printable and non-7-bit ASCII characters. */
	for (c = s.begin(); c != s.end();) {
		if ((unsigned int)*c < 0x20 || (unsigned int)*c >= 0x7f) {
			s.erase(c);
			ret++;
			continue;
		}
		c++;
	}

	if (s.find("&amp;", 0) < s.size())
		appears_sanitized = true;

	/*
	 * Escape ampersands to prevent incomplete XML entity references. Must
	 * not be done for attribute content, where the `&' is supposed to show
	 * up literally.
	 */
	if (!for_attribute && !appears_sanitized) {
		size_t	i = 0;
		do {
			if ((i = s.find('&', i)) < s.size())
				s.insert(++i, "amp;");
		} while (i < s.size());
	}

	/*
	 * Escape quotation marks and apostrophes if we're sanitizing for
	 * attributes.
	 */
	if (for_attribute) {
		size_t	i = 0, j = 0;
		do {
			if ((i = s.find('"', i)) < s.size())
				s.replace(i, 1, "&quot;");
			if ((j = s.find('\'', j)) < s.size())
				s.replace(j, 1, "&apos;");
		} while (i < s.size() || j < s.size());
	}

#if 0
	cout << "ATT [" << (for_attribute ? "X" : " ") << "]  SAN ["
		<< (appears_sanitized ? "X" : " ") << "]: " << s << endl;
#endif
	return (ret);
}

/*
 * Removes excess whitespaces from, and then sanitizes a string.
 */
void
XSI_Util::normalizeString(string &s, bool for_attribute)
{
	string			tmp;
	string::iterator	c;
	bool			is_space;

	if (s.empty())
		return;

	is_space = true;
	for (c = s.begin(); c != s.end(); c++) {
		if (*c == ' ') {
			if (!is_space && tmp.size() > 0 &&
			    tmp[tmp.size() - 1] != ' ')
				tmp += " ";
			is_space = true;
		} else {
			is_space = false;
			tmp += *c;
		}
	}
	if (tmp.size() > 0 && tmp[tmp.size() - 1] == ' ')
		tmp.resize(tmp.size() - 1);

	sanitizeString(tmp, for_attribute);

	s = tmp;
}

vector<string>
XSI_Util::split(string str, char separator)
{
	string			tmp;
	string::iterator	c;
	vector<string>		ret;
	bool			is_separator;

	is_separator = true;
	for (c = str.begin(); c != str.end(); c++) {
		/* be paranoid; control chars are separators, too */
		if (*c == separator || *c < 0x20) {
			if (!is_separator && !tmp.empty())
				ret.push_back(tmp);
			is_separator = true;
			tmp = "";
		} else {
			is_separator = false;
			tmp += *c;
		}
	}
	if (!tmp.empty())
		ret.push_back(tmp);

	return (ret);
}

vector<string>
XSI_Util::getFileVector(const char *file, bool *must_exist)
{
	vector<string>	ret;
	string		line;
	ifstream	f;

	f.open(file, ifstream::in);
	if (!f.is_open()) {
		if (must_exist != NULL) {
			if (*must_exist)
				XSI_Log::log()->err(__func__, errno, "Error opening %s", file);
			else {
				*must_exist = true;
				return (ret);
			}
		} else
			XSI_Log::log()->err(__func__, errno, "Error opening %s", file);
	}


	while ((int)f.peek() != EOF) {
		line.push_back(f.get());
		if (line.size() > 0 && line[line.size() - 1] == '\n') {
			line.resize(line.size() - 1);	// trim trailing newline
			ret.push_back(line);
			line = "";
		}
	}
	if (!line.empty() && !line[0] == '\n')
		ret.push_back(line);

	f.close();

	return (ret);
}

string
XSI_Util::getFileString(const char *file, bool *must_exist)
{
	string		ret;
	ifstream	f;

	f.open(file, ifstream::in);
	if (!f.is_open()) {
		if (must_exist != NULL) {
			if (*must_exist)
				XSI_Log::log()->err(__func__, errno, "Error opening %s", file);
			else {
				*must_exist = true;
				return (ret);
			}
		} else
			XSI_Log::log()->err(__func__, errno, "Error opening %s", file);
	}

	while (f.peek() != '\n' && (int)f.peek() != EOF) {
		ret.push_back(f.get());
	}

	return (ret);
}

int64_t
XSI_Util::getValueFromFile(const char *file, int64_t min, int64_t max)
{
	string		s = getFileString(file);
	const char     *errstr;
	int64_t 	ret;

	ret = strtonum(s.c_str(), min, max, &errstr);
	if (errstr)
		XSI_Log::log()->err(__func__, "in %s: %s is %s",
				    file, s.c_str(), errstr);
	return (ret);
}

int64_t
XSI_Util::getValueFromFile(const char *file, bool *error, int64_t min, int64_t max)
{
	string		s;
	const char     *errstr;
	int64_t 	ret;
	bool		exist = false;

	s = getFileString(file, &exist);
	if (exist) {
		*error = true;
		return (0);
	}

	ret = strtonum(s.c_str(), min, max, &errstr);
	if (errstr)
		*error = true;
	else
		*error = false;
	return (ret);
}
