/*	$Kolabore: util.h,v 1.14 2007/02/09 20:58:49 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * util.h
 * Generic utility and helper methods for internal XSI use
 */

#ifndef __UTIL_H__
#define __UTIL_H__

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include "compat/typesetup.h"

extern "C" {
#include <sys/types.h>

#ifdef HAVE_GOOD_STDINT_H
#  include <stdint.h>
#endif
}

#include <string>
#include <vector>

using namespace std;

class XSI_Util {
public:
	const char *	makeFloatStr(int64_t, double divisor = 1,
				     unsigned short precision = 2);
	const char *	makeFloatStr(uint64_t, double divisor = 1,
				     unsigned short precision = 2);
	const char *	makeFloatStr(double, double divisor = 1,
				     unsigned short precision = 2);
	const char *	makeIntStr(int64_t);
	const char *	makeIntStr(uint64_t);
	const char *	makeIntStr(int32_t);
	const char *	makeIntStr(uint32_t);

	unsigned long	sanitizeString(string &, bool for_attribute = false);
	void		normalizeString(string &, bool for_attribute = false);

	vector<string>	split(string, char separator = ' ');
	vector<string>	getFileVector(const char *, bool *must_exist = NULL);
	string		getFileString(const char *, bool *must_exist = NULL);
	int64_t 	getValueFromFile(const char *, int64_t min = INT64_MIN,
					 int64_t max = INT64_MAX);
	int64_t 	getValueFromFile(const char *, bool *,
					 int64_t min = INT64_MIN,
					 int64_t max = INT64_MAX);
};

#endif /* __UTIL_H__ */
