/*	$Kolabore: xmlsysinfo.h,v 1.46 2007/01/09 18:34:51 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __XMLSYSINFO_H__
#define __XMLSYSINFO_H__

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

extern "C" {
#include <libxml/tree.h>
}

#include "cache.h"
#include "cfg.h"
#include "system.h"
#include "network.h"
#include "sensor.h"
#include "storage.h"
#include "util.h"

/*
 * XML Schema version; only loosely coupled with XSI versioning.
 * The XSD URI may be defined elsewhere, e.g. to point to a local copy.
 */
#define XSI_XSD_VERSION 	"1.0-beta"
#ifndef XSI_XSD_NAMESPACE
#  define XSI_XSD_NAMESPACE	"http://xmlsysinfo.sf.net"
#endif /* XSI_XSD_NAMESPACE */
#ifndef XSI_XSD_URI
#  define XSI_XSD_URI		"http://xmlsysinfo.sourceforge.net/XSD/xmlsysinfo-1.0.xsd"
#endif /* XSI_XSD_URI */
#define XSI_XSD_NS_URI_STRING   XSI_XSD_NAMESPACE " " XSI_XSD_URI

using namespace std;

/*
 * XSIBase class, the root of all evil.
 * Has class XSI_* member variables whose methods can be used to set
 * information and statistics
 */
class XSIBase : protected XSI_Util {
public:
	XSIBase(void);
	virtual ~XSIBase(void);

	void			XSIBase_init(Config *);
	void			setListenAddrs(vector<string>);
	void			setListenService(string);
	void			readSysInfo(void);
	void			updateSysInfo(void);
	void			buildSysInfoDoc(void);
	xmlDocPtr		getSysInfoDocPtr(void);
	const char *		getUserCacheStatsString(void);
	void			resetCaches(void);

	/*
	 * XXX (Because it's not so nice.)
	 * Solaris' libpicl uses callback functions, and this is for a simple
	 * workaround until something better comes along. External access to
	 * the sensors vector is required.
	 */
#if defined(SOLARIS)
	vector<XSI_Sensor>	sensors;
#endif /* SOLARIS */

private:
	friend class XSI_TimeKeeper;

	void			updateTimeKeeper(void);

protected:
	virtual int		tryOpenPrivilegedDevices(void);
	virtual int		initializePacketfilters(void);
	virtual void		updatePacketfilterStats(void);
	virtual void		collectCommonInfo(void);
	virtual void		updateCommonStats(void);

	/* System dependent code typically implements only the following: */
	virtual void		collectSysInfo(void);
	virtual void		updateStats(void);


	Config		       *configuration;
	xmlDocPtr		sysInfoDocument;
	vector<string>		listenAddrs;
	string			listenService;

	XSI_UserCache	       *user_cache;

	XSI_System		system;
	XSI_Top 		proc_info;
#if !defined(SOLARIS)
	vector<XSI_Sensor>	sensors;
#endif /* !SOLARIS */
	XSI_Storage		storage;
	vector<XSI_Raid>	raids;
	XSI_Network		network;
	vector<XSI_PFilter>	packetFilters;
};

#endif /* __XMLSYSINFO_H__ */
