/*	$Kolabore: fuzzer.cpp,v 1.27 2007/01/10 19:10:20 maxx Exp $	*/
/*
 * Copyright (c) 2006, 2007 Moritz Grimm <gtgbr@gmx.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * fuzzer.cpp:
 * Code to fill the output with random garbage, i.e. "FuzzOS support".
 */

#include "log.h"

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <netinet/in.h>

#include <netdb.h>
}

#include <string>
#include <vector>

#include <cstdlib>
#include <cstring>
#include <ctime>

#include "fuzzer.h"

using namespace std;

/*
 * XSI Constructor
 *
 * !! WARNING !!
 * !! Runs before dropping privileges when running as root
 * !! WARNING !!
 */
XSI::XSI(void)
{
	// empty
}

XSI::~XSI(void)
{
	// empty
}

/*
 * Silently returns 0 on success. Warns and returns >0 on error.
 */
int
XSI::tryOpenPrivilegedDevices(void)
{
	return (0);
}

int
XSI::initializePacketfilters(void)
{
	return (0);
}

void XSI::updatePacketfilterStats(void) {}
void XSI::collectCommonInfo(void) {}
void XSI::updateCommonStats(void) {}

/*
 * Information read once on startup
 */
void
XSI::collectSysInfo(void)
{
	num_cpu = getRandomInt32(true) % 24;
	system.setNumCPUs(num_cpu);
}

/*
 * Stats to be updated regularly
 */
void
XSI::updateStats(void)
{
	setSystemInfo();
	setProcInfo();
	setSensorInfo();
	setStorageInfo();
	setRAIDInfo();
	setNetworkInfo();
	setPacketFilterInfo();
}

void
XSI::setSystemInfo(void)
{
	vector<struct cpu>	cpus;
	int			i;
	struct vm_info		vm;
	string			tmp;
	double			load[3];
	vector<XSI_Interrupt>	intrs = system.getInterrupts();

	if (getRandomBool() && getRandomBool() && getRandomBool()) {
		num_cpu = getRandomInt32(true) % 24;
		system.setNumCPUs(num_cpu);
	}

	system.setModel(getRandomString().c_str());
	system.setCPUFreq(getRandomInt32(true));
	system.setVendor(getRandomString().c_str());
	system.setProduct(getRandomString().c_str());
	system.setVersion(getRandomString().c_str());
	system.setSerialNo(getRandomString().c_str());
	system.setUUID(getRandomString().c_str());
	if (getRandomBool()) {
		for (i = 0; i < num_cpu; i++) {
			struct cpu	cpu;

			cpu.num = i;
			cpu.system    = getRandomPercent();
			cpu.user      = getRandomPercent();
			cpu.nice      = getRandomPercent();
			cpu.interrupt = getRandomPercent();
			cpu.idle      = getRandomPercent();

			cpus.push_back(cpu);
		}
	} else {
		struct cpu      cpu;

		cpu.num = -1;
		cpu.system    = getRandomPercent();
		cpu.user      = getRandomPercent();
		cpu.nice      = getRandomPercent();
		cpu.interrupt = getRandomPercent();
		cpu.idle      = getRandomPercent();

		cpus.push_back(cpu);
	}
	system.setCPUs(cpus);
	vm.ram_total = (uint64_t)getRandomInt64();
	vm.ram_free = vm.ram_total - (uint64_t)getRandomInt64() % vm.ram_total;
	vm.swap_total = (uint64_t)getRandomInt64();
	vm.swap_free = vm.swap_total - (uint64_t)getRandomInt64() % vm.swap_total;
	system.setVMInfo(vm);

	if (getRandomBool() && getRandomBool()) {
		int	num_int_srcs;

		intrs.clear();
		num_int_srcs = getRandomInt32(true) % 32;
		for (i = 0; i < num_int_srcs; i++) {
			XSI_Interrupt intr(getRandomString(1, 64, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());

			intr.setIntsCount((uint64_t)getRandomInt64());
			intrs.push_back(intr);
		}
	} else {
		vector<XSI_Interrupt>::iterator it;

		for (it = intrs.begin(); it != intrs.end(); it++)
			it->setIntsCount((uint64_t)getRandomInt64());
	}
	system.setInterrupts(intrs);

	tmp = getRandomString(0, 32, FUZZ_DEFAULT | FUZZ_NOSPACE);
	tmp += "#";
	tmp += getRandomString(1, 6, FUZZ_DIGITSONLY);
	tmp += " ";
	if (getRandomBool())
		tmp += getRandomString(12, 12, FUZZ_DIGITSONLY);
	else {
		tmp += "p";
		if (getRandomBool())
			tmp += "0";
		else
			tmp += getRandomString(6, 7, FUZZ_DIGITSONLY);
	}
	system.setOSType("FuZzOS");
	system.setOSInfo(getRandomString(0, 32, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str(),
			 tmp.c_str(),
			 getRandomString().c_str());

	system.setOSUptime(getRandomInt32(true));
	for (i = 0; i < 3; i++)
		load[i] = getRandomDecimal();
	system.setLoadAvg(load);
}

void
XSI::setProcInfo(void)
{
	int				i, num_procs;
	vector<struct process_info>	procs;

	num_procs = getRandomInt32(true) % 512;
	for (i = 0; i < num_procs; i++) {
		struct process_info p;

		p.pid = xsi_random();
		p.name = getRandomString();
		p.euid = xsi_random();
		p.username = user_cache->getUsername(p.euid);
		p.login_user = getRandomString();
		p.running = getRandomBool();
		p.zombie = getRandomBool();
		p.pctcpu = getRandomPercent();
		p.ressize = (uint64_t)getRandomInt64();
		p.nice = (xsi_random() % 40) - 20;

		procs.push_back(p);
	}
	proc_info.setProcessVector(procs);
}

void
XSI::setSensorInfo(void)
{
	int     i, num_sensors;

	sensors.clear();

	num_sensors = getRandomInt32(true) % 64;
	for (i = 0; i < num_sensors; i++) {
		XSI_Sensor s;
		enum xsi_sensor_type type;

		s.setIsValid(getRandomBool());
		s.setHasUnknownValue(getRandomBool());
		s.setSensorID(xsi_random() % 256);
		s.setSensorDevice(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
		type = (enum xsi_sensor_type)((getRandomInt32(true) % ((int)XSI_SENSOR_TYPE_MAX - 1)) + 1);
		s.setSensorType(type);
		if (type == XSI_SENSOR_TYPE_DRIVE)
			s.setSensorValue((enum xsi_drive_sensor_status)(1 + getRandomInt32(true) % ((int)XSI_DRIVE_STATUS_MAX - 1)));
		else
			s.setSensorValue(getRandomInt64(),
					 (double)(1 + (getRandomInt32(true) % 1000000000)),
					 getRandomInt32(true) % 10);
		s.setSensorDescription(getRandomString().c_str());
		s.setOtherSensorType(getRandomString(1, 32, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
		s.setOtherSensorUnit(getRandomString(1, 32, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
		s.setSensorStatus((enum xsi_sensor_status)(getRandomInt32(true) % (int)XSI_SENSOR_STATUS_MAX));
		if (getRandomBool()) {
			struct timeval tv;

			tv.tv_sec = (long)getRandomInt32(true);
			tv.tv_usec = (long)getRandomInt32(true);

			s.setValueLastChangeTime(&tv);
		} else
			s.setValueAge(getRandomDecimal());

		sensors.push_back(s);
	}
}

void
XSI::setStorageInfo(void)
{
	vector<struct fs>	fsystems;
	vector<XSI_DiskIO>	diovect = storage.getDisks();
	int			i, num_fs, num_dsk;

	num_fs = 1 + getRandomInt32(true) % 24;
	num_dsk = getRandomInt32(true) % 12;

	for (i = 0; i < num_fs; i++) {
		struct fs	fs;

		fs.mountpoint = getRandomString();
		fs.fs_name = getRandomString(1, 32, FUZZ_DEFAULT | FUZZ_NOSPACE);
		fs.fs_type = getRandomString(1, 32, FUZZ_DEFAULT | FUZZ_NOSPACE);
		fs.size_total = (uint64_t)getRandomInt64();
		fs.size_free = fs.size_total - (uint64_t)getRandomInt64() % fs.size_total;
		fs.size_avail = fs.size_free - (uint64_t)getRandomInt64() % fs.size_free;
		fs.nodes_total = (uint64_t)getRandomInt64();
		fs.nodes_free = fs.nodes_total - (uint64_t)getRandomInt64() % fs.nodes_total;

		fsystems.push_back(fs);
	}
	storage.setFilesystems(fsystems);

	if (getRandomBool() && getRandomBool())
		diovect.clear();

	if (diovect.empty()) {
		for (i = 0; i < num_dsk; i++) {
			XSI_DiskIO d(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());

			d.setWrittenBytes((uint64_t)getRandomInt64());
			d.setReadBytes((uint64_t)getRandomInt64());
			d.setWriteTransfers((uint64_t)getRandomInt64());
			d.setReadTransfers((uint64_t)getRandomInt64());

			diovect.push_back(d);
		}
	} else {
		vector<XSI_DiskIO>::iterator	it;

		for (it = diovect.begin(); it != diovect.end(); it++) {
			it->setWrittenBytes((uint64_t)getRandomInt64());
			it->setReadBytes((uint64_t)getRandomInt64());
			it->setWriteTransfers((uint64_t)getRandomInt64());
			it->setReadTransfers((uint64_t)getRandomInt64());
		}
	}
	storage.setDisks(diovect);
}

void
XSI::setRAIDInfo(void)
{
	int			i, num_raid_ctrls;
	vector<XSI_Raid>	rtmp;

	num_raid_ctrls = 1 + getRandomInt32(true) % 4;

	for (i = 0; i < num_raid_ctrls; i++) {
		XSI_Raid	r(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
		int		j, num_vols, num_rdsks;
		vector<XSI_RVolume> vols;
		vector<XSI_RDisk> rdsks;

		num_vols = getRandomInt32(true) % 5;
		for (j = 0; j < num_vols; j++) {
			XSI_RVolume v;
			int	k, num_vdsks;
			vector<XSI_RDisk> vdsks;
			enum xsi_raid_vol_status status;

			v.setVolumeDevName(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
			v.setVolumeID(j);
			v.setRaidLevel(getRandomInt32(true) % 21);
			status = (enum xsi_raid_vol_status)(getRandomInt32(true) % (int)XSI_RAID_VOL_MAX);
			v.setVolumeStatus(status);
			if (status == XSI_RAID_VOL_BUILDING ||
			    status == XSI_RAID_VOL_SCRUBBING ||
			    status == XSI_RAID_VOL_REBUILD) {
				v.setVolumeOpProgress(getRandomInt32(true));
				v.setVolumeOpElapsed(getRandomInt32(true));
			}
			v.setVolumeSize((uint64_t)getRandomInt64());

			num_vdsks = 1 + getRandomInt32(true) % 8;
			for (k = 0; k < num_vdsks; k++) {
				XSI_RDisk d;

				d.setDiskID(k);
				d.setDiskLocation(getRandomInt32(true) % 64,
						  getRandomInt32(true) % 64);
				d.setDiskLUN(getRandomInt32(true) % 64);
				d.setDiskStatus((enum xsi_raid_disk_status)(getRandomInt32(true) % (int)XSI_RAID_DSK_MAX));
				d.setDiskSize((uint64_t)getRandomInt64());
				d.setDiskEnclosureDevName(getRandomString(0, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
				d.setDiskVendorName(getRandomString().c_str());
				d.setDiskSerial(getRandomString().c_str());

				vdsks.push_back(d);
			}
			v.setVolumeDisks(vdsks);

			vols.push_back(v);
		}
		r.setVolumes(vols);

		num_rdsks = getRandomInt32(true) % 9;
		for (j = 0; j < num_rdsks; j++) {
			XSI_RDisk d;

			d.setDiskID(j);
			d.setDiskLocation(getRandomInt32(true) % 64,
					  getRandomInt32(true) % 64);
			d.setDiskLUN(getRandomInt32(true) % 64);
			d.setDiskStatus((enum xsi_raid_disk_status)(getRandomInt32(true) % (int)XSI_RAID_DSK_MAX));
			d.setDiskSize((uint64_t)getRandomInt64());
			d.setDiskEnclosureDevName(getRandomString(0, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
			d.setDiskVendorName(getRandomString().c_str());
			d.setDiskSerial(getRandomString().c_str());

			rdsks.push_back(d);
		}
		r.setReserveDisks(rdsks);

		rtmp.push_back(r);
	}

	raids = rtmp;
}

void
XSI::setNetworkInfo(void)
{
	vector<struct default_route> def_rts;
	vector<XSI_NetIfStat> nstats = network.getNetIfStats();
	int	i, num_defrts, num_netifs;

	num_defrts = getRandomInt32(true) % 7;
	num_netifs = getRandomInt32(true) % 9;

	for (i = 0; i < num_defrts; i++) {
		struct default_route rt;

		if (getRandomBool()) {
			struct sockaddr_in	addr;
			char			buf[BUFSIZ];
			int			error;
			string			rnd = getRandomString(sizeof(addr),
								      sizeof(addr),
								      FUZZ_NASTY);

			memcpy(&addr, rnd.data(), sizeof(addr));
			addr.sin_family = AF_INET;
			if ((error = getnameinfo((struct sockaddr *)&addr,
						 sizeof(addr), buf, sizeof(buf),
						 NULL, 0, NI_NUMERICHOST)) != 0)
				XSI_Log::log()->err(__func__, "getnameinfo: %s",
						    gai_strerror(error));

			rt.family = XSI_RTAF_IPV4;
			rt.gateway = buf;
		} else {
			struct sockaddr_in6	addr;
			char			buf[BUFSIZ];
			int			error;
			string			rnd = getRandomString(sizeof(addr),
								      sizeof(addr),
								      FUZZ_NASTY);

			memcpy(&addr, rnd.data(), sizeof(addr));
			addr.sin6_family = AF_INET6;
			if ((error = getnameinfo((struct sockaddr *)&addr,
						 sizeof(addr), buf, sizeof(buf),
						 NULL, 0, NI_NUMERICHOST)) != 0)
				XSI_Log::log()->err(__func__, "getnameinfo: %s",
						    gai_strerror(error));

			rt.family = XSI_RTAF_IPV6;
			rt.gateway = buf;
		}

		def_rts.push_back(rt);
	}
	network.setDefaultRoutes(def_rts);

	if (getRandomBool() && getRandomBool())
		nstats.clear();

	if (nstats.empty()) {
		for (i = 0; i < num_netifs; i++) {
			XSI_NetIfStat	n(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str(),
					  getRandomBool());
			string		tmp;
			int		j, num_addrs;
			vector<XSI_IfAddress> addrs;

			n.setIsRunning(getRandomBool());
			n.setIsPromiscuous(getRandomBool());
			n.setIsLoopback(getRandomBool());
			n.setDescription(getRandomString().c_str());
			n.setMediaString(getRandomString().c_str());
			if (getRandomBool()) {
				tmp = "";
				for (j = 0; j < 6; j++) {
					tmp += getRandomString(2, 2, FUZZ_HEXDIGITS);
					if (j < 5)
						tmp += ":";
				}
				n.setLladdrString(tmp.c_str());
			}
			n.setMetric(xsi_random());
			n.setMTU(xsi_random());
			n.setLinkStatus((enum if_link_status)(getRandomInt32(true) % (int)IF_LINK_MAX));
			n.setDuplexStatus((enum if_duplex_status)(getRandomInt32(true) % (int)IF_DUPLEX_MAX));
			n.setPacketsIn((uint64_t)getRandomInt64());
			n.setPacketsOut((uint64_t)getRandomInt64());
			n.setPacketsDropped((uint64_t)getRandomInt64());
			n.setErrorsIn((uint64_t)getRandomInt64());
			n.setErrorsOut((uint64_t)getRandomInt64());
			n.setBytesIn((uint64_t)getRandomInt64());
			n.setBytesOut((uint64_t)getRandomInt64());
			n.setCollisions((uint64_t)getRandomInt64());
			if (getRandomBool()) {
				if (getRandomBool()) {
					struct sockaddr_in addr;
					string	rnd = getRandomString(sizeof(addr),
								      sizeof(addr),
								      FUZZ_NASTY);

					memcpy(&addr, rnd.data(), sizeof(addr));
					addr.sin_family = AF_INET;

					n.setPTPDestAddress((struct sockaddr *)&addr, sizeof(addr));
				} else {
					struct sockaddr_in6 addr;
					string	rnd = getRandomString(sizeof(addr),
								      sizeof(addr),
								      FUZZ_NASTY);

					memcpy(&addr, rnd.data(), sizeof(addr));
					addr.sin6_family = AF_INET6;

					n.setPTPDestAddress((struct sockaddr *)&addr, sizeof(addr));
				}
			}
			num_addrs = getRandomInt32(true) % 9;
			for (j = 0; j < num_addrs; j++) {
				XSI_IfAddress a;

				if (getRandomBool() && getRandomBool()) {
					a.setAddressString(getRandomString(1, 32, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str(),
							  (int)AF_INET6 + (xsi_random() % AF_MAX - (int)AF_INET6));
				} else {
					if (getRandomBool()) {
						struct sockaddr_in addr, mask;
						string  rnd = getRandomString(sizeof(addr),
									      sizeof(addr),
									      FUZZ_NASTY);

						memcpy(&addr, rnd.data(), sizeof(addr));
						rnd = getRandomString(sizeof(addr),
								      sizeof(addr),
								      FUZZ_NASTY);
						memcpy(&mask, rnd.data(), sizeof(mask));
						addr.sin_family = mask.sin_family = AF_INET;
						a.setAddress((struct sockaddr *)&addr, sizeof(addr),
							     (struct sockaddr *)&mask, AF_INET);
					} else {
						struct sockaddr_in6 addr, mask;
						string  rnd = getRandomString(sizeof(addr),
									      sizeof(addr),
									      FUZZ_NASTY);

						memcpy(&addr, rnd.data(), sizeof(addr));
						rnd = getRandomString(sizeof(addr),
								      sizeof(addr),
								      FUZZ_NASTY);
						memcpy(&mask, rnd.data(), sizeof(mask));
						addr.sin6_family = mask.sin6_family = AF_INET6;
						a.setAddress((struct sockaddr *)&addr, sizeof(addr),
							     (struct sockaddr *)&mask, AF_INET6);
					}
				}

				addrs.push_back(a);
			}
			n.setIfAddresses(addrs);
			n.setCARPStatus(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str(),
					(enum xsi_carp_state)(1 + getRandomInt32(true) % ((int)CARP_MAX - 1)),
					xsi_random(), xsi_random(), xsi_random(),
					getRandomBool(), getRandomBool());

			nstats.push_back(n);
		}
	} else {
		vector<XSI_NetIfStat>::iterator it;

		for (it = nstats.begin(); it != nstats.end(); it++) {
			it->setPacketsIn((uint64_t)getRandomInt64());
			it->setPacketsOut((uint64_t)getRandomInt64());
			it->setPacketsDropped((uint64_t)getRandomInt64());
			it->setErrorsIn((uint64_t)getRandomInt64());
			it->setErrorsOut((uint64_t)getRandomInt64());
			it->setBytesIn((uint64_t)getRandomInt64());
			it->setBytesOut((uint64_t)getRandomInt64());
			it->setCollisions((uint64_t)getRandomInt64());
		}
	}
	network.setNetIfStats(nstats);
}

void
XSI::setPacketFilterInfo(void)
{
	int	i;

	for (i = 0; i < XSI_PFILTER_MAX; i++) {
		vector<FilteredIfStat> fifs;

		packetFilters[i].setPFilterType((enum xsi_pfilter_type)i);

		if (getRandomBool() && getRandomBool()) {
			packetFilters[i].setAvailable(false);
			continue;
		}

		packetFilters[i].setAvailable(true);
		packetFilters[i].setEnabled(getRandomBool());
		packetFilters[i].setRuntime(getRandomInt32(true));
		packetFilters[i].setRuleMatches((uint64_t)getRandomInt64());
		packetFilters[i].setStates((uint64_t)getRandomInt32());
		packetFilters[i].setStatesLimit(getRandomInt64());
		packetFilters[i].setStateSearches((uint64_t)getRandomInt64());
		packetFilters[i].setStateInserts((uint64_t)getRandomInt64());
		packetFilters[i].setStateRemovals((uint64_t)getRandomInt64());
		packetFilters[i].setStateMismatches((uint64_t)getRandomInt64());

		if (getRandomBool()) {
			int	j, num_fifs;

			num_fifs = getRandomInt32(true) % 9;
			for (j = 0; j < num_fifs; j++) {
				FilteredIfStat st(getRandomString(1, 24, FUZZ_DEFAULT | FUZZ_NOSPACE).c_str());
				bool bytecounters = false;

				st.setPktsV4OutBlocked((uint64_t)getRandomInt64());
				st.setPktsV4OutPassed((uint64_t)getRandomInt64());
				st.setPktsV4InBlocked((uint64_t)getRandomInt64());
				st.setPktsV4InPassed((uint64_t)getRandomInt64());
				if (getRandomBool()) {
					bytecounters = true;
					st.setBytesV4OutBlocked((uint64_t)getRandomInt64());
					st.setBytesV4OutPassed((uint64_t)getRandomInt64());
					st.setBytesV4InBlocked((uint64_t)getRandomInt64());
					st.setBytesV4InPassed((uint64_t)getRandomInt64());
				}
				if (getRandomBool())
					st.setIPv4IsSummaryTraffic(true);
				else {
					st.setIPv4IsSummaryTraffic(false);
					st.setPktsV6OutBlocked((uint64_t)getRandomInt64());
					st.setPktsV6OutPassed((uint64_t)getRandomInt64());
					st.setPktsV6InBlocked((uint64_t)getRandomInt64());
					st.setPktsV6InPassed((uint64_t)getRandomInt64());
					st.setBytesV6OutBlocked((uint64_t)getRandomInt64());
					st.setBytesV6OutPassed((uint64_t)getRandomInt64());
					st.setBytesV6InBlocked((uint64_t)getRandomInt64());
					st.setBytesV6InPassed((uint64_t)getRandomInt64());
				}
				fifs.push_back(st);
			}
		} else {
			FilteredIfStat st(NULL);
			bool bytecounters = false;

			st.setPktsV4OutBlocked((uint64_t)getRandomInt64());
			st.setPktsV4OutPassed((uint64_t)getRandomInt64());
			st.setPktsV4InBlocked((uint64_t)getRandomInt64());
			st.setPktsV4InPassed((uint64_t)getRandomInt64());
			if (getRandomBool()) {
				bytecounters = true;
				st.setBytesV4OutBlocked((uint64_t)getRandomInt64());
				st.setBytesV4OutPassed((uint64_t)getRandomInt64());
				st.setBytesV4InBlocked((uint64_t)getRandomInt64());
				st.setBytesV4InPassed((uint64_t)getRandomInt64());
			}
			if (getRandomBool())
				st.setIPv4IsSummaryTraffic(true);
			else {
				st.setIPv4IsSummaryTraffic(false);
				st.setPktsV6OutBlocked((uint64_t)getRandomInt64());
				st.setPktsV6OutPassed((uint64_t)getRandomInt64());
				st.setPktsV6InBlocked((uint64_t)getRandomInt64());
				st.setPktsV6InPassed((uint64_t)getRandomInt64());
				st.setBytesV6OutBlocked((uint64_t)getRandomInt64());
				st.setBytesV6OutPassed((uint64_t)getRandomInt64());
				st.setBytesV6InBlocked((uint64_t)getRandomInt64());
				st.setBytesV6InPassed((uint64_t)getRandomInt64());
			}
			fifs.push_back(st);
		}
		packetFilters[i].setFilteredIfStats(fifs);
	}
}

uint16_t
XSI::xsi_random(void)
{
#if defined(HAVE_ARC4RANDOM)
	return ((uint16_t)arc4random());
#elif defined(HAVE_RANDOM)
	static bool first_run = true;

	if (first_run) {
		srandom(time(NULL));
		first_run = false;
	}

	return ((uint16_t)random());
#else
	static bool first_run = true;

	if (first_run) {
		srand(time(NULL));
		first_run = false;
	}

	/* Drop the lower dozen bits, which go through a cyclic pattern. */
	return ((uint16_t)(rand() >> 12));
#endif /* HAVE_ARC4RANDOM */
}

int32_t
XSI::getRandomInt32(bool nonnegative)
{
	unsigned int	i;
	char		num[4];
	int32_t 	ret;

	for (i = 0; i < sizeof(num); i++)
		num[i] = xsi_random() & 0xff;
	ret = *(int32_t *)num;

	if (nonnegative && ret < 0)
		return (-ret);
	else
		return (ret);
}

int64_t
XSI::getRandomInt64(bool nonnegative)
{
	unsigned int	i;
	char		num[8];
	int64_t 	ret;

	for (i = 0; i < sizeof(num); i++)
		num[i] = xsi_random() & 0xff;
	ret = *(int64_t *)num;

	if (nonnegative && ret < 0)
		return (-ret);
	else
		return (ret);
}

double
XSI::getRandomPercent(void)
{
	return ((double)(((uint64_t)getRandomInt64() % 100000001) / 1000000.0));
}

double
XSI::getRandomDecimal(void)
{
	return ((double)(((uint64_t)getRandomInt64() % UINT64_C(10000000000000000001)) / 100000000.0));
}

string
XSI::getRandomString(unsigned int min_len,
		     unsigned int max_len,
		     int fuzz)
{
	unsigned int	i, len;
	string		ret = "";
	char		c;

	if (max_len < min_len)
		XSI_Log::log()->err(__func__,
				    "Internal error: max_len is smaller than min_len");

	if (min_len == max_len)
		len = max_len;
	else
		len = min_len + (xsi_random() % (max_len - min_len + 1));

	for (i = 0; i < len; i++) {
		do {
			c = (char)(xsi_random() & 0xff);
		} while (((fuzz & FUZZ_PRINTABLE) && (unsigned int)c < 0x20) ||
			 ((fuzz & FUZZ_7BITPRINT) &&
			  (int)c < 0x20 && (int)c != 0x7f) ||
			 ((fuzz & FUZZ_NOSPACE) && c == ' ') ||
			 ((fuzz & FUZZ_DIGITSONLY) && (c < '0' || c > '9')) ||
			 ((fuzz & FUZZ_HEXDIGITS) &&
			  ((c < '0' || c > '9') && (c < 'a' || c > 'f'))) ||
			 c == '\0');
		ret.push_back(c);
	}

	return (ret);
}

bool
XSI::getRandomBool(void)
{
	if ((xsi_random() % 1024) < 512)
		return (true);
	else
		return (false);
}
